"""rtypes: main type hiearchy

The type system is a hierarchical system, where each child type adds some
information w.r.t to its parent type. 

Root type is TypeUnknown, with as child TypeAny. Remaining types are divided in
- TypeScalar:   all scalar types, such as integers, strings.
- TypeTuple:    values containing multiple attributes/fields/elements, each
                with a (possibly different) type. 
- TypeArray:    collection of values of similar type, in an storage structure 
                described by dimensions
- TypeFunc:     methods/functions (to do)

.. rubric:: Adding new types
    Adding a new type is as simple as inheriting from one of the types,
    and calling the function addType(>typeclass<), e.g. 

    TypeDNASequence(TypeString):
        name = "DNASequence"
    addType(TypeDNASequence)

    Extra attributes such as _dtype, _scalar can be set to adapt the 
    internal numpy representation. Examples can be found in the source code of 
    this file. 

    When the type is added, one can add operations (type_ops.py),
    casts (type_casts.py) and automatic object detection (scanners.py).


.. rubric:: Hierarchy rules
    One should only inherit if the child type can only contain a subset 
    of the values of the parent type. 

    Operations on the parent type should be applicable on the child type 
    (but one can override them for more type-specific behaviour).

    Multiple inheritance is allowed, e.g. TypeUInt8(TypeUInt16, TypeInt16)

    It is currently not possible to add types within the hieararchy (e.g.
    a type inbetween TypeUnknown and any of the other types), 
    without modifying the source in this file or some hackish runtime class modification. 
"""


import platform
import copy
import numpy
import operator
from collections import defaultdict

from ..constants import *
from ..utils import util
from ..parser_objs import *
_delay_import_(globals(),"dimensions")
_delay_import_(globals(),"dimpaths")
_delay_import_(globals(),"casts")
_delay_import_(globals(),"type_attribute_freeze")
_delay_import_(globals(),"..utils.missing","Missing")

#}}}

#typename to type class dictionary
__typenames__ = {}
#children types classes for each parent type class
__typechildren__ = defaultdict(list)

from ibidas import constants

def addType(newtypecls):
    """Adds a type `newtypecls`
    to the type hiearchy"""

    __typenames__[newtypecls.name] = newtypecls
    for basecls in newtypecls.__bases__:
        __typechildren__[basecls].append(newtypecls)

class Type(object):#{{{
    """Base type class. Represents the type of a data structure.
    """
    name = "?"
    _dtype = "object"
    _scalar = numpy.object
    _defval = None
    _reqRPCcon=True
    has_missing = True

    @classmethod
    def commonType(cls, type1, type2):
        """Returns a common supertype for type1 and type2"""
        return unknown

    def _requiresRPCconversion(self):
        return self._reqRPCcon

    def toNumpy(self):
        """Returns numpy dtype compatible with this type

        :rtype: numpy dtype
        """
        return numpy.dtype(self._dtype)

    def toScalar(self):
        """Returns numpy scalar classs compatible with this type
        
        :rtype: numpy scalar class
        """
        return self._scalar

    def toDefval(self):
        """Returns default value."""
        return self._defval
  
    def toMissingval(self):
        return Missing
    
    def hasMissingValInfo(self):
        try:
           self.toMissingval()
           return True
        except RuntimeError:
           return False

    def hasMissing(self):
        return self.has_missing

    def getName(self):
        """Returns base type name"""
        return self.name

    def _callSubtypes(self, methodname, *params, **kwds):
        """If has subtype, calls subtype with methodname and params

        :param methodname: Name of type method. Should return type object
        :param params: arguments for ``methodname``.
        :param kwds: Keyword arguments for ``methodname``
        """
        return self

    #dim changes
    def _removeDepDim(self, pos, elem_specifier):
        return self._callSubtypes("_removeDepDim",pos, elem_specifier)

    def _updateDepDim(self, pos, ndim):
        return self._callSubtypes("_updateDepDim",pos, ndim)

    def _insertDepDim(self, pos, ndim):
        return self._callSubtypes("_insertDepDim",pos, ndim)

    def _permuteDepDim(self, prevdims, permute_idxs):
        return self._callSubtypes("_permuteDepDim",prevdims, permute_idxs)

    #subtypes
    def getSubTypeNumber(self):
        """Returns number of subtypes of this type"""
        return 0

    def getSubType(self, subtype_id=0):
        """Returns subtype if this is a non-scalar type.
        Otherwise, raises TypeError. If subtype_id invalid, raised IndexError.

        :param subtype_id: id of subtype, default 0. Should be >= 0 and < :py:meth:`getSubTypeNumber`.
        :rtype: obj of class :py:class:`Type`"""
        return unknown
    
    #comparison
    def __eq__(self, other):
        if(isinstance(other, basestring)):
            other = createType(other)
        return other.__class__ is self.__class__
    
    def __ne__(self, other):
        return not self.__eq__(other)

    def __le__(self, other):
        return self.__eq__(other)
    
    def __ge__(self, other):
        return True

    def __gt__(self, other):
        return not self.__eq__(other)
    
    def __lt__(self, other):
        return False

    def __hash__(self):
        return hash(self.__class__)

    def __repr__(self):
        return self.name
    
    def copy(self):
        """Returns copy of this type"""
        return self
   
    @classmethod
    def getDescendantTypes(cls):
        """Returns descendant type classes as list

        :rtype: list of descendant type classes"""
        if(not hasattr(cls, "_desc_types")):
            desc_types = [tcls.getDescendantTypes() 
                                for tcls in __typechildren__[cls]]
            #flatten list (concat each list in list)
            if(desc_types):
                desc_types = sum(desc_types, [])
            desc_types.append(cls)
            cls._desc_types = desc_types
        return cls._desc_types

class TypeUnknown(Type):
    """Unknown type represents the lack of information about
    the actual type. 

    As this type has only one possible state, 
    a module singleton is available through rtypes.unknown
    """
addType(TypeUnknown)#}}}

#unknown singleton
unknown = TypeUnknown()

class TypeAny(TypeUnknown):#{{{
    """Type which represents that any value is allowed"""

    name = "any"
    _ptype = lambda x : x

    def __init__(self, has_missing=False):
        """
        Creates type object.

        :param has_missing: bool, optional
        """
        TypeUnknown.__init__(self)
        self.has_missing = has_missing
        
    @classmethod
    def commonType(cls, type1, type2):
        return cls(type1.has_missing or type2.has_missing)
   
    def toNumpy(self):
        """Returns dtype of a numpy container which
        can hold this type efficiently.
        """
        if(self.has_missing):
            return numpy.dtype(object)
        else:
            return numpy.dtype(self._dtype)
    
    def getSubType(self, subtype_id=0):
        """Returns subtype if this is a non-scalar type.
        Otherwise, raises TypeError. If subtype_id invalid, raised IndexError.

        :param subtype_id: id of subtype, default 0. Should be >= 0 and < :py:meth:`getSubTypeNumber`.
        :rtype: obj of class :py:class:`Type`"""
        raise TypeError, "Expected subtypeable type, but found " + str(self)
    
    def getPythonType(self):
        return self._ptype

    def __eq__(self, other):
        if(isinstance(other, basestring)):
            other = createType(other)
        return (self.__class__ is other.__class__ and 
                self.has_missing is other.has_missing)
    
    def __ne__(self, other):
        return not self.__eq__(other)

    def __le__(self, other):
        if(isinstance(other, basestring)):
            other = createType(other)
        if(other.__class__ == self.__class__):
            if(self.has_missing):
                return other.has_missing
            return True
        if(other.__class__ in self.__class__.__mro__):
            return True
        return False
    
    def __ge__(self, other):
        if(isinstance(other, basestring)):
            other = createType(other)
        if(other.__class__ == self.__class__):
            if(other.has_missing):
                return self.has_missing
            return True
        if(self.__class__ in other.__class__.__mro__):
            return True
        return False

    def __gt__(self, other):
        if(isinstance(other, basestring)):
            other = createType(other)
        if(other.__class__ == self.__class__):
            return self.has_missing and not other.has_missing
        if(self.__class__ in other.__class__.__mro__):
            return True
        return False
    
    def __lt__(self, other):
        if(isinstance(other, basestring)):
            other = createType(other)
        if(other.__class__ == self.__class__):
            return not self.has_missing and other.has_missing
        if(other.__class__ in self.__class__.__mro__):
            return True
        return False

    def __hash__(self):
        return hash(self.__class__) ^ hash(self.has_missing)

    def copy(self, **newattr):
        """Returns copy of this type"""
        return copy.copy(self)

    def setHasMissing(self, value):
        if self.has_missing != value:
            res = self.copy()
            res.has_missing = value
        else:
            res = self
        return res

    def __repr__(self):
        res = self.name
        if(self.has_missing):
            res += "?"
        return res
addType(TypeAny)#}}}

class TypeTuple(TypeAny):#{{{
    """Tuple or record type, having multiple values of 
       possibly different types"""
    name = "tuple"
    _ptype = tuple

    def __init__(self, has_missing=False, subtypes=(), 
                       fieldnames=()):
        """
        Creates type object.

        :param has_missing: bool, optional
        :param subtypes: tuple(type objects, i.e. subclasses of TypeUnknown), optional
        :param fieldnames: tuple(strings), optional
        """

        assert isinstance(subtypes, tuple), \
                "The subtypes argument should be a tuple"

        assert all(isinstance(fieldname, basestring) for fieldname in fieldnames), \
                "Fieldnames should be strings"

        assert not fieldnames or len(fieldnames) == len(subtypes), \
            "Length of fieldnames should be equal to length of tuples (or empty"

        TypeAny.__init__(self, has_missing)
        self.subtypes = subtypes
        self.fieldnames = fieldnames
    
    @classmethod
    def commonType(cls, type1, type2):
        if(not type1.subtypes or not type2.subtypes or
            len(type1.subtypes) != len(type2.subtypes)):
            return cls(type1.has_missing or type2.has_missing)
        else:
            subtypes = [casts.castImplicitCommonType(lstype, rstype)
                      for lstype, rstype in zip(type1.subtypes, type2.subtypes)]
            if(False in subtypes):
                return False
            res = cls(type1.has_missing or type2.has_missing, tuple(subtypes), type1.fieldnames) 
        return res
    
    def toDefval(self):
        """Returns default value."""

        if(not "_defval" in self.__dict__):
            self._defval = tuple((subtype.toDefval() 
                                  for subtype in self.subtypes))
        return self._defval
   
    def getSubTypeNumber(self):
        """Returns number of subtypes of this type"""
        return len(self.subtypes)

    def getSubType(self, subtype_id=0):
        """Returns subtype if this is a non-scalar type.
        Otherwise, raises TypeError. If subtype_id invalid, raised IndexError.

        :param subtype_id: id of subtype, default 0. Should be >= 0 and < :py:meth:`getSubTypeNumber`.
        :rtype: obj of class :py:class:`Type`"""
        if(self.subtypes):
            return self.subtypes[subtype_id]
        else:
            return unknown

    
    def __eq__(self, other):
        if(isinstance(other, basestring)):
            other = createType(other)
        return (self.__class__ is other.__class__ and 
                self.has_missing is other.has_missing and 
                self.subtypes == other.subtypes and 
                self.fieldnames == other.fieldnames)
 
    def __ne__(self, other):
        return not self.__eq__(other)

    def __le__(self, other):
        if(isinstance(other, basestring)):
            other = createType(other)
        if(other.__class__ == self.__class__):
            if(self.has_missing and not other.has_missing):
                return False
            if((not self.subtypes and other.subtypes) or (other.subtypes and
                len(self.subtypes) != len(other.subtypes))):
                return False
            if(not all([st <= ot for st, ot in 
                            zip(self.subtypes, other.subtypes)])):
                return False
            return True
        if(other.__class__ in self.__class__.__mro__):
            return True
        return False
    
    def __ge__(self, other):
        if(isinstance(other, basestring)):
            other = createType(other)
        if(other.__class__ == self.__class__):
            if(other.has_missing and not self.has_missing):
                return False
            if((not other.subtypes and self.subtypes) or (self.subtypes and
                len(self.subtypes) != len(other.subtypes))):
                return False
            if(not all([st >= ot for st, ot in 
                            zip(self.subtypes, other.subtypes)])):
                return False
            return True
        if(self.__class__ in other.__class__.__mro__):
            return True
        return False

    def __gt__(self, other):
        if(isinstance(other, basestring)):
            other = createType(other)
        if(other.__class__ == self.__class__):
            if(other.has_missing and not self.has_missing):
                return False
            if(not other.subtypes or (self.subtypes and 
                len(self.subtypes) != len(other.subtypes))):
                return False
            if(not all([st > ot for st, ot in 
                            zip(self.subtypes, other.subtypes)])):
                return False
            return True
        if(self.__class__ in other.__class__.__mro__):
            return True
        return False
    
    def __lt__(self, other):
        if(isinstance(other, basestring)):
            other = createType(other)
        if(other.__class__ == self.__class__):
            if(self.has_missing and not other.has_missing):
                return False
            if(not self.subtypes or (other.subtypes and
                len(self.subtypes) != len(other.subtypes))):
                return False
            if(not all([st < ot for st, ot in 
                            zip(self.subtypes, other.subtypes)])):
                return False
            return True
        if(other.__class__ in self.__class__.__mro__):
            return True
        return False

    def __hash__(self):
        return (hash(self.__class__) ^ 
                hash(self.has_missing) ^ 
                hash(self.subtypes) ^
                hash(self.fieldnames))
    
    def _callSubtypes(self, methodname, *params, **kwds):
        nsubtypes = tuple([getattr(subtype,methodname)(*params,**kwds) for subtype in self.subtypes])

        if(all([nsubtype is subtype for nsubtype,subtype in zip(nsubtypes,self.subtypes)])):
            nself = self
        else:
            nself = self.copy()
            nself.subtypes = nsubtypes
        return nself

    def __repr__(self):
        res = '(' 
        if(len(self.fieldnames) == len(self.subtypes)):
            res += ", ".join((fname + "=" + str(subtype) 
                for fname, subtype in zip(self.fieldnames, self.subtypes)))
        else:
            res += ", ".join((str(subtype) 
                    for subtype in self.subtypes))
        res += ')'
        if(self.has_missing):
            res += "?"
        return res
addType(TypeTuple)#}}}

class TypeRecordDict(TypeTuple):#{{{
    name = "record_dict"
    _ptype = None

    def __repr__(self):
        res = '{' 
        if(len(self.fieldnames) == len(self.subtypes)):
            res += ", ".join((fname + "=" + str(subtype) 
                for fname, subtype in zip(self.fieldnames, self.subtypes)))
        else:
            res += ", ".join((str(subtype) 
                    for subtype in self.subtypes))
        res += '}'
        if(self.has_missing):
            res += "?"
        return res

addType(TypeRecordDict)#}}}

class TypeIndexDict(TypeTuple):#{{{
    name = "index_dict"
    _ptype = None
    
    def __init__(self, has_missing=False, subtypes=(), 
                       fieldnames=("key","value")):
        """
        Creates type object.

        :param has_missing: bool, optional
        :param subtypes: tuple(type objects, i.e. subclasses of TypeUnknown), optional
        :param fieldnames: tuple(strings), optional
        """

        assert isinstance(subtypes, tuple), \
                "The subtypes argument should be a tuple"

        assert all(isinstance(fieldname, basestring) for fieldname in fieldnames), \
                "Fieldnames should be strings"

        assert not fieldnames or len(fieldnames) == len(subtypes), \
            "Length of fieldnames should be equal to length of tuples (or empty"

        assert len(subtypes) == 2, "IndexDict should have a key and value subtype"

        TypeAny.__init__(self, has_missing)
        self.subtypes = subtypes
        self.fieldnames = fieldnames

    def __repr__(self):
        res = 'dict(' 
        if(len(self.fieldnames) == len(self.subtypes)):
            res += ": ".join((fname + "=" + str(subtype) 
                for fname, subtype in zip(self.fieldnames, self.subtypes)))
        else:
            res += ": ".join((str(subtype) 
                    for subtype in self.subtypes))
        res += ')'
        if(self.has_missing):
            res += "?"
        return res

addType(TypeIndexDict)#}}}

#pylint: disable-msg=E1101
class TypeArray(TypeAny):#{{{
    """Type representing a collection of values, 
       possibly in an dimensional structure"""
    name = "array"
    _ptype = list

    def __init__(self, has_missing=False, dims=(), \
                       subtypes=(unknown,)):
        """
        Creates type object.

        :param has_missing: bool, optional
        :param dims: tuple(Dim's), optional
        :param subtypes: tuple(type object, i.e. subclass of TypeUnknown), optional

        A dimid is a unique identifier for a dimension, helping the
        system to determine equality between dimensions. If there are
        no similarities with dimensions in other types, should be left empty.
        """
        assert isinstance(dims, dimpaths.DimPath), \
                "Dims of an array should be a dimpath"
        assert len(dims) == 1, "Array should have one dimension"
        assert all([isinstance(dim, dimensions.Dim) for dim in dims]), \
                "Dims tuple should contain Dim objects"
        assert isinstance(subtypes, tuple) and len(subtypes) == 1, \
                "One subtype should be set for array type"
        
        has_missing = dims[0].has_missing or has_missing
        self.subtypes = subtypes
        self.dims = dims
        TypeAny.__init__(self, has_missing)
    
    @classmethod
    def commonType(cls, type1, type2):
        if(not type1.dims or not type2.dims or
            len(type1.dims) != len(type2.dims)):
            return TypeAny(type1.has_missing or type2.has_missins)
        else:
            subtypes = [casts.castImplicitCommonType(lstype, rstype)
                        for lstype, rstype in zip(type1.subtypes, type2.subtypes)]
            if(False in subtypes):
                return False
            ndims = []
            for ldim, rdim in zip(type1.dims, type2.dims):
                ndims.append(ldim.merge(rdim))

            dims = dimpaths.DimPath(*ndims)
            res = cls(has_missing=type1.has_missing or type2.has_missing, dims=dims, subtypes=tuple(subtypes)) 
        return res

    def setHasMissing(self, value):
        if value and not self.dims[0].dependent: #fixed dim, needs has_missing subtype if unpacked
            s = self.subtypes[0].setHasMissing(value)
        else:
            s = self.subtypes[0]
            
        if not self.has_missing == value or not s is self.subtypes[0]:
            self = self.copy()
            self.has_missing = value
            self.subtypes = (s,)
            
        return self


    def toMissingval(self):
        s = self.subtypes[0].toMissingval()
        if self.dims[0].dependent:
            res = numpy.array([],dtype=self.subtypes[0].toNumpy())
        else:
            if self.dims[0].shape == UNDEFINED:
                raise RuntimeError, "Cannot determine shape for missing value. Please cast dim " + str(self.dims[0])
            res = numpy.array([s] * self.dims[0].shape,dtype=self.toNumpy())
        return res

    def hasMissing(self):
        return self.subtypes[0].hasMissing()


    def toDefval(self):
        """Returns default value."""
        subtype = self.subtypes[0]
        shape = [dim.shape for dim in self.dims]
        for pos, sdim in enumerate(shape):
            if(sdim < 0):
                shape[pos] = 0
        
        res = numpy.empty(shape, dtype=subtype.toNumpy())
        subdv = subtype.toDefval()
        
        #workaround numpy problem 
        #(cannot set sequence for a range)
        if(operator.isSequenceType(subdv)):
            flatres = res.ravel()
            for i in range(len(flatres)):
                flatres[i] = subdv
        else:
            res[:] = subdv
        return res
    

    def getSubTypeNumber(self):
        """Returns number of subtypes of this type"""
        return len(self.subtypes)

    def getSubType(self, subtype_id=0):
        """Returns subtype if this is a non-scalar type.
        Otherwise, raises TypeError. If subtype_id invalid, raised IndexError.

        :param subtype_id: id of subtype, default 0. Should be >= 0 and < :py:meth:`getSubTypeNumber`.
        :rtype: obj of class :py:class:`Type`"""
        assert (subtype_id == 0), "Invalid subtype id given"
        if(self.subtypes):
            return self.subtypes[subtype_id]
        else:
            return unknown
    
    def __eq__(self, other):
        if(isinstance(other, basestring)):
            other = createType(other)
        return (self.__class__ is other.__class__ and 
                self.has_missing is other.has_missing and 
                self.subtypes == other.subtypes and 
                self.dims == other.dims)
 
    def __ne__(self, other):
        return not self.__eq__(other)

    def __le__(self, other):
        if(isinstance(other, basestring)):
            other = createType(other)
        if(other.__class__ == self.__class__):
            if(self.has_missing and not other.has_missing):
                return False
            if((not self.subtypes and other.subtypes) or
                len(self.subtypes) != len(other.subtypes)):
                return False
            if(not all([st <= ot for st, ot in 
                            zip(self.subtypes, other.subtypes)])):
                return False
            return all([d1.shape == d2.shape for d1, d2 in 
                                zip(self.dims, other.dims)])
        if(other.__class__ in self.__class__.__mro__):
            return True
        return False
    
    def __ge__(self, other):
        if(isinstance(other, basestring)):
            other = createType(other)
        if(other.__class__ == self.__class__):
            if(other.has_missing and not self.has_missing):
                return False
            if((not other.subtypes and self.subtypes) or
                len(self.subtypes) != len(other.subtypes)):
                return False
            if(not all([st >= ot for st, ot in 
                            zip(self.subtypes, other.subtypes)])):
                return False
            return all([d1.shape == d2.shape for d1, d2 in 
                                    zip(self.dims, other.dims)])
        if(self.__class__ in other.__class__.__mro__):
            return True
        return False

    def __gt__(self, other):
        if(isinstance(other, basestring)):
            other = createType(other)
        if(other.__class__ == self.__class__):
            if(other.has_missing and not self.has_missing):
                return False
            if(not other.subtypes or
                len(self.subtypes) != len(other.subtypes)):
                return False
            if(not all([st > ot for st, ot in 
                            zip(self.subtypes, other.subtypes)])):
                return False
            return all([d1.shape == d2.shape for d1, d2 in 
                                    zip(self.dims, other.dims)])
        if(self.__class__ in other.__class__.__mro__):
            return True
        return False
    
    def __lt__(self, other):
        if(isinstance(other, basestring)):
            other = createType(other)
        if(other.__class__ == self.__class__):
            if(self.has_missing and not other.has_missing):
                return False
            if(not self.subtypes or
                len(self.subtypes) != len(other.subtypes)):
                return False
            if(not all([st < ot for st, ot in 
                            zip(self.subtypes, other.subtypes)])):
                return False
            return all([d1.shape == d2.shape for d1, d2 in 
                                    zip(self.dims, other.dims)])
        if(other.__class__ in self.__class__.__mro__):
            return True
        return False

    def __hash__(self):
        return (hash(self.__class__) ^ 
                hash(self.has_missing) ^ 
                hash(self.subtypes) ^ 
                hash(self.dims))
    
    def _callSubtypes(self, methodname, *params, **kwds):
        nsubtypes = tuple([getattr(subtype,methodname)(*params,**kwds) for subtype in self.subtypes])

        if(all([nsubtype is subtype for nsubtype,subtype in zip(nsubtypes,self.subtypes)])):
            nself = self
        else:
            nself = self.copy()
            nself.subtypes = nsubtypes
        return nself
   
    def _removeDepDim(self, pos, elem_specifier):
        nself = self._callSubtypes("_removeDepDim",pos - len(self.dims), elem_specifier)
        ndims = self.dims.removeDim(pos,elem_specifier)
        if(not ndims is self.dims):
            if(self is nself):
                nself = self.copy()
            nself.dims = ndims
        return nself
     
    def _updateDepDim(self, pos, ndim):
        nself = self._callSubtypes("_updateDepDim", pos - len(self.dims), ndim)
        ndims = self.dims.updateDim(pos,ndim)
        if(not ndims is self.dims):
            if(self is nself):
                nself = self.copy()
            nself.dims = ndims
        return nself

    def _insertDepDim(self, pos, ndim):
        nself = self._callSubtypes("_insertDepDim",pos - len(self.dims), ndim)
        ndims = self.dims.insertDim(pos, ndim)
        if(not ndims is self.dims):
            if(self is nself):
                nself = self.copy()
            nself.dims = ndims
        return nself

    def _permuteDepDim(self, prevdims, permute_idxs):
        nself = self._callSubtypes("_permuteDepDim", prevdims + self.dims, permute_idxs)
        ndims = self.dims.permuteDims(permute_idxs, prevdims=prevdims)
        if(not ndims is self.dims):
            if(self is nself):
                nself = self.copy()
            nself.dims = ndims
        return self

    def __repr__(self, unpack_depth=0):
        
        res = '[' + ",".join([str(dim) for dim in self.dims]) + ']'
        if(self.has_missing):
            res += "?"
        if(unpack_depth > 1):
            assert isinstance(self.subtypes, TypeArray), \
                   "Unpack depth of tuple is larger than number of " + \
                   "array subtypes. Found: " + str(self.subtypes[0])
            res += '<' + self.subtypes[0].__repr__(unpack_depth - 1)
        else:
            res += ':' + str(self.subtypes[0]) 
        return res
addType(TypeArray)#}}}

class TypeSet(TypeArray):#{{{
    name = "set"
    _ptype = set

    def __init__(self, has_missing=False, dims=(), subtypes=(unknown,)):

        assert (isinstance(dims, dimpaths.DimPath) and len(dims) == 1), \
                "Dimensions of a set should be a dimpath of size 1"

        assert subtypes and len(subtypes) == 1 and isinstance(subtypes,tuple), \
                 "Number of subtypes should be 1"
        
        TypeArray.__init__(self, has_missing, dims, 
                                 subtypes)
    def __repr__(self):
        res = ""
        if(self.has_missing):
            res += "?"
        res += '{' + ",".join([str(dim) for dim in self.dims]) + '}'
        res += '<' + str(self.subtypes[0]) 
        return res
    
    def hasMissing(self):
        return self.has_missing

    def toMissingval(self):
        return frozenset()

addType(TypeSet)#}}}

class TypeString(TypeArray):#{{{
    """String type (supports unicode strings)"""
    name = "string"
    _dtype = "U"
    _defval = u""
    _ptype = unicode
    _reqRPCcon=False
    
    def __init__(self, has_missing=False, dims=()):
        assert (isinstance(dims, dimpaths.DimPath) and len(dims) == 1), \
            "Dimensions of a string should be a dimpath of size 1"
        TypeArray.__init__(self, has_missing, dims, 
                                    (TypeChar(),))

    @classmethod
    def commonType(cls, type1, type2):
        if(type1.dims[0].shape == UNDEFINED or
            type2.dims[0].shape == UNDEFINED):
            shape = UNDEFINED
        else:
            shape = max(type1.dims[0].shape, type2.dims[0].shape)
        dim = dimensions.Dim(shape)
        res = cls(has_missing=type1.has_missing or type2.has_missing, 
                  dims=dimpaths.DimPath(dim))
        return res


    def toNumpy(self):
        """Returns dtype of a numpy container which
           can hold this type efficiently."""

        return numpy.dtype(object)
        #f(self.dims[0].shape == UNDEFINED or self.has_missing or self.dims[0].shape > 32):
        #   return numpy.dtype(object)
        #lse:
        #   return numpy.dtype(self._dtype + str(max(self.dims[0].shape,1)))
    
    def __eq__(self, other):
        return (self.__class__ is other.__class__ and 
                self.has_missing is other.has_missing and \
                self.dims[0].shape == other.dims[0].shape)

    def __hash__(self):
        return (hash(self.__class__) ^ 
                hash(self.has_missing) ^ 
                hash(self.dims[0].shape))

    def toMissingval(self):
        return Missing
    
    def hasMissing(self):
        return self.has_missing
   
    def __repr__(self):
        res =  self.name
        if(self.has_missing):
            res += "?"
        if(self.dims[0].shape >= 0):
            res += '[' + str(self.dims[0].shape) + ']'
        return res


    def toDefval(self):
        """Returns default value."""
        return ""

addType(TypeString)#}}}

class TypeBytes(TypeString):#{{{
    """String type (only byte/ascii characters)"""
    name = "bytes"
    _dtype = "S"
    _defval = b""
    _ptype = bytes
addType(TypeBytes)#}}}

class TypePickle(TypeBytes):#{{{
    name = "pickle"
addType(TypePickle)#}}}

class TypeScalar(TypeAny):#{{{
    """Type representing atom-like values"""
    name = "scalar"

addType(TypeScalar)#}}}

class TypeChar(TypeScalar):#{{{
    """Type representing characters"""
    name = "char"
    
addType(TypeChar)#}}}

class TypeSlice(TypeScalar):#{{{
    """Type representing slices"""
    name = "slice"
    _ptype = slice
    
addType(TypeSlice)#}}}

class TypeNumber(TypeScalar):#{{{
    """Type representing the number values"""
    name = "number"
    _defval = 0
    
    def __init__(self, has_missing=False):
        TypeScalar.__init__(self, has_missing)
    
addType(TypeNumber)#}}}

class TypeComplex(TypeNumber):#{{{
    """Complex number"""
    name = "complex"
    _defval = 0.0j
    
    def toNumpy(self):
        """Returns dtype of a numpy container which
        can hold this type efficiently.
        """
        if(self.has_missing):
            return numpy.dtype(object)
        else:
            return numpy.dtype(self._dtype)
    
addType(TypeComplex)#}}}

class TypeComplex128(TypeComplex):#{{{
    """Complex number, representable by machine doubles"""
    name = "complex128"
    _dtype = "complex128"
    _scalar = numpy.complex128
addType(TypeComplex128)#}}}

class TypeComplex64(TypeComplex128):#{{{
    """Complex number, representable by machine singles"""

    name = "complex64"
    _dtype = "complex64"
    _scalar = numpy.complex64
addType(TypeComplex64)#}}}

class TypeReal64(TypeComplex128):#{{{
    """Floating point number, representable by machine double"""
    name = "real64"
    _dtype = "float64"
    _scalar = numpy.float64
    _defval = 0.0
    _reqRPCcon=False
    _ptype = float
addType(TypeReal64)#}}}

class TypeReal32(TypeReal64, TypeComplex64):#{{{
    """Floating point number, representable by machine single"""
    name = "real32"
    _dtype = "float32"
    _scalar = numpy.float32
addType(TypeReal32)#}}}

class TypeInteger(TypeReal32):#{{{
    """Integer number"""
    name = "long"
    _dtype = "object"
    _minmax = (-numpy.inf, numpy.inf)
    _ptype = int 
    @classmethod
    def getMinValue(cls):
        """Returns minimum integer that can be stored 
        with this type"""
        return cls._minmax[0]

    @classmethod
    def getMaxValue(cls):
        """Returns maximum integer that can be stored
        with this type"""
        return cls._minmax[1]
    
    def toNumpy(self):
        """Returns dtype of a numpy container which
        can hold this type efficiently.
        """
        if(self.has_missing):
            return numpy.dtype(object)
        else:
            return numpy.dtype(self._dtype)

addType(TypeInteger)#}}}

class TypeInt64(TypeInteger):#{{{
    """Integer number, 
    range -9,223,372,036,854,775,808 to +9,223,372,036,854,775,807"""
    name = "int64"
    _minmax = (-2**63, 2**63-1)
    _dtype = "int64"
    _scalar = numpy.int64
addType(TypeInt64)#}}}

class TypeInt32(TypeInt64):#{{{
    """Integer number,
    range -2,147,483,648 to +2,147,483,647"""

    name = "int32"
    _minmax = (-2**31, 2**31-1)
    _dtype = "int32"
    _scalar =  numpy.int32
addType(TypeInt32)#}}}

class TypeInt16(TypeInt32):#{{{
    """Integer number,
    range -32,768 to +32,767"""

    name = "int16"
    _minmax = (-2**15, 2**15-1)
    _dtype = "int16"
    _scalar = numpy.int16
addType(TypeInt16)#}}}

class TypeInt8(TypeInt16):#{{{
    """Integer number,
    range -128 to 127"""

    name = "int8"
    _minmax = (-2**7, 2**7-1)
    _dtype = "int8"
    _scalar = numpy.int8
addType(TypeInt8)#}}}

class TypeUnsignedInteger(TypeInteger):#{{{
    """Integer number (no negative integers)"""
    name = "ulong"
addType(TypeUnsignedInteger)#}}}

class TypeUInt64(TypeUnsignedInteger):#{{{
    """Integer number,
    range 0 to 18,446,744,073,709,551,615"""
    name = "uint64"
    _minmax = (0, 2**64-1)
    _dtype = "uint64"
    _scalar = numpy.uint64
addType(TypeUInt64)#}}}

class TypeUInt32(TypeUInt64, TypeInt64):#{{{
    """Integer number,
    range 0 to 4,294,967,295"""
    name = "uint32"
    _minmax = (0, 2**32-1)
    _dtype = "uint32"
    _scalar = numpy.uint32
addType(TypeUInt32)#}}}

class TypeUInt16(TypeUInt32, TypeInt32):#{{{
    """Integer number,
    range 0 to 65,535"""
    name = "uint16"
    _dtype = "uint16"
    _minmax = (0, 2**16-1)
    _scalar = numpy.uint16
addType(TypeUInt16)#}}}

class TypeUInt8(TypeUInt16, TypeInt16):#{{{
    """Integer number,
    range 0 to 255"""

    name = "uint8"
    _minmax = (0, 2**8-1)
    _dtype = "uint8"
    _scalar = numpy.uint8
addType(TypeUInt8)#}}}

class TypeBool(TypeUInt8, TypeInt8):#{{{
    """Integer number,
    range 0 to 1 (or False, True)"""

    name = "bool"
    _minmax = (0, 1)
    _dtype = "bool"
    _scalar = numpy.bool
    _defval = False
    _reqRPCcon=True
    
addType(TypeBool)#}}}


#maps numpy type to internal type
__objtype_map__ = {
numpy.dtype("object"):TypeAny,
numpy.dtype("int8"):TypeInt8,
numpy.dtype("int16"):TypeInt16,
numpy.dtype("int32"):TypeInt32,
numpy.dtype("int64"):TypeInt64,
numpy.dtype("uint8"):TypeInt8,
numpy.dtype("uint16"):TypeInt16,
numpy.dtype("uint32"):TypeInt32,
numpy.dtype("uint64"):TypeInt64,
numpy.dtype("float32"):TypeReal32,
numpy.dtype("float64"):TypeReal64,
numpy.dtype("complex64"):TypeComplex64,
numpy.dtype("complex128"):TypeComplex128}



NO_NAME = 0
NAME_FOUND = 1
INDIM = 2
INSUBTYPES = 3

NAME_FOUND_ARRAY = 2
IN_SUBTYPE = 2
EXIT_SUBTYPE = 3
IN_DIM = 4
OUT_DIM = 5
IN_SUBTYPE_DIM = 6




def createType(name, dimpos=0, refdims=[], env={}):#{{{
    """Creates a type object from string representation.

    :param name: str

    Format:
        Description of type formats

    Examples:
        >>> createType("unicode")

        >>> createType("array(int64)[]")

        >>> createType("tuple(int64,unicode)")

    """

    if(not isinstance(name, basestring)):
        if(not isinstance(name, numpy.dtype)):
            name = numpy.dtype(name)
        
        if(name.char == 'S'):
            if(name.itemsize == 0):
                dim = dimensions.Dim(UNDEFINED, (True,) * dimpos, False) 
                return TypeBytes(dims=dimpaths.DimPath(dim))
            else:
                dim = dimensions.Dim(name.itemsize, (True,) * dimpos, False) 
                return TypeBytes(dims=dimpaths.DimPath(dim))
        elif(name.char == 'U'):
            if(name.itemsize == 0):
                dim = dimensions.Dim(UNDEFINED, (True,) * dimpos, False) 
                return TypeString(dims=dimpaths.DimPath(dim))
            else:
                usize = numpy.array("test").dtype.itemsize / 4
                dim = dimensions.Dim(name.itemsize / usize, (True,) * dimpos, False) 
                return TypeString(dims = dimpaths.DimPath(dim))
        elif(name in __objtype_map__):
            return __objtype_map__[name](False)
        else:
            raise TypeError,"Unknown type description: " + str(name)

    return _createType(name, dimpos, refdims, env)#}}}

class TypeStringScanner(GenericScanner):#{{{
    def tokenize(self, input):
        self.rv = []
        GenericScanner.tokenize(self,input)
        return self.rv

    def t_whitespace(self,s):
        r' \s+ '
        pass

    def t_name(self,s):
        r' [a-zA-Z_][a-zA-Z_\d]* '
        t = Token(type='name',attr=s)
        self.rv.append(t)

    def t_integer(self,s):
        r' \d+ '
        t = Token(type="integer",attr=int(s))
        self.rv.append(t)

    def t_symbol(self,s):
        r' \= | \$ | \? | \. | \{ | \} | \< | \[ | \] | \( | \) | \, | \* | \~ | \! | \& | \# | \@ | \: | \^ | \| | \' '
        t = Token(type=s)
        self.rv.append(t)#}}}
    
class TypeStringParser(GenericParser):#{{{
    def __init__(self, start="typenest"):
        GenericParser.__init__(self, start)

    def p_param_1(self,args):
        '   param ::= @ name '
        return AST(type="param",kids=args[1:])

    def p_tmatcher(self, args):
        """
            tmatcher ::= typenest
            tmatcher ::= ^ typenest
        """
        if len(args) > 1:
            return AST(type="tmatcher",kids=args)
        else:
            return args[0]

    def p_tmatcher_list(self, args):
        """
            type_orlist ::= tmatcher
            type_orlist ::= type_orlist | tmatcher
        """
        return AST(type="type_orlist",kids=args[:1] + args[2:])

    def p_type_match_0(self, args):
        """
            type_andlist ::= type_orlist
            type_andlist ::= type_andlist & type_orlist
        """
        return AST(type="type_andlist", kids=args[:1] + args[2:])
   
    def p_type_andlist_1(self, args):
        """
            typenest ::= # type_andlist #
        """
        return AST(type="typenest", kids=args[1:2])

    def p_dmatcher(self, args):
        """
            dmatcher ::= nameddim
            dmatcher ::= ^ nameddim
        """
        if len(args) > 1:
            return AST(type="dmatcher",kids=args)
        else:
            return args[0]

    def p_dmatcher_list(self, args):
        """
            dim_orlist ::= dmatcher
            dim_orlist ::= dim_orlist | dmatcher
        """
        return AST(type="dim_orlist",kids=args[:1] + args[2:])
    
    def p_dim_match_0(self, args):
        """
            dim_andlist ::= dim_orlist
            dim_andlist ::= dim_andlist & dim_orlist
        """
        return AST(type="dim_andlist", kids=args[:1] + args[2:])

    def p_dim_andlist(self, args):
        """
            nameddim ::= # dim_andlist #
        """
        return AST(type="nameddim", kids=args[1:2])

    def p_param_call(self, args):
        """
            param ::= param ( typelist )
            param ::= param ( dimlist )
        """
        return AST(type="param_call", kids=[args[0], args[2]])

    def p_type_1(self,args):
        """
            type ::= param
            type ::= name 
            type ::= ?
        """
        return AST(type="createtype",kids=args[:1])
    
    def p_type_2(self,args):
        """
            type ::= type $
            type ::= type ?
            type ::= type !
        """
        return AST(type="hasmissing", kids=args)
    
    def p_type_2b(self,args):
        """
            type ::= type '
        """
        return AST(type="strict", kids=args)
   
    def p_type_3(self,args):
        ' type ::= type [ dimlist ] '
        return AST(type="dims",kids=(args[0], args[2]))
    
    def p_type_3b(self,args):
        ' type ::= type [ ] '
        return AST(type="dims",kids=(args[0],))

    def p_type_4(self,args):
        ' type ::= type ( typelist ) '
        return AST(type="subtypes",kids=(args[0], args[2]))
    
    def p_type_6(self,args):
        ' type ::= [ dimlist ] '
        return AST(type="dims",kids=(
            AST(type="createtype", kids=(Token(type="name",attr="array"),)), args[1]))
    
    def p_type_7(self,args):
        ' type ::= { dimlist } '
        return AST(type="dims",kids=(
            AST(type="createtype", kids=(Token(type="name",attr="set"),)), args[1]))
    
    def p_type_8(self,args):
        '''
            typeelem ::= typenest
            typeelem ::= name = typenest
        '''
        if len(args) == 1:
            return args[0]
        else:
            return AST(type="typeelem",kids=(args[2], args[0]))
    
    def p_type_9(self,args):
        ' type ::= ( typelist ) '
        return AST(type="subtypes",kids=(
            AST(type="createtype", kids=(Token(type="name",attr="tuple"),)), args[1]))
    
    def p_type_9b(self,args):
        ' type ::= { typelist } '
        return AST(type="subtypes",kids=(
            AST(type="createtype", kids=(Token(type="name",attr="record_dict"),)), args[1]))

    
    def p_typenest_2(self,args):
        """
        typenest ::= type
        typenest ::= typenest < type
        typenest ::= typenest : type
        typenest ::= typenest < typenest
        typenset ::= typenest : typenest
        """
        return AST(type="typenest",kids=args)

    def p_var_1(self,args):
        ''' 
            var ::= .
            var ::= *
        '''
        return args[0]
        
    def p_varlist_1(self,args):
        ''' 
            varlist ::= var
        '''
        return AST(type="varlist",kids=args[:1])
    
    def p_varlist_2(self,args):
        ''' 
            varlist ::= varlist var
        '''
        return AST(type="varlist",kids=args)


    def p_dim_1(self,args):
        ''' 
            dim ::= integer 
            dim ::= ~
            dim ::= varlist
            dim ::= param
        '''
        return AST(type="createdim",kids=args)


    def p_nameddim_1(self,args):
        ''' nameddim ::= name : dim '''
        return AST(type="namedim",kids=(args[2], args[0]))
    
    def p_nameddim_2(self,args):
        ''' nameddim ::= dim '''
        return args[0]
    
    def p_nameddim_3(self,args):
        ''' nameddim ::= name '''
        return AST(type="namedim",kids=(
            AST(type="createdim", kids=(Token(type="inherit"),)), args[0]))
   
    def p_nameddim_4(self,args):
        ''' 
            nameddim ::= nameddim  $
            nameddim ::= nameddim  ?
            nameddim ::= nameddim  !
        '''
        return AST(type="hasmissing",kids=args)

    def p_dim_6(self, args):
        """
            dimelem ::= nameddim
            dimelem ::= ?
            dimelem ::= ? ?
        """
        if(len(args) > 1):
            ntoken = Token(type="??")
        else:
            ntoken = args[0]
        return ntoken

    def p_dimlist_1(self,args):
        ''' 
            dimlist ::= dimelem 
            dimlist ::= dimlist , dimelem 
        '''
        return AST(type="dimlist",kids=args[:1] + args[2:])

    def p_typelist_1(self,args):
        ''' typelist ::= typeelem 
            typelist ::= typelist , typeelem '''
        return AST(type="typelist", kids=(args[:1] + args[2:]))#}}}

class TypeStringASTRewriterPass1(GenericASTRewriter):#{{{
    def process(self, tree):
        self.dim_annot = {}
        return (self.postorder(tree), self.dim_annot)
    
    def collapse_list(listname):
        def func(self, node):
            if(len(node.kids) > 1):
                if node.kids[0].type == listname:
                    node.kids = tuple(node.kids[0].kids) + (node.kids[1],)
                return node 
            else:
                return node.kids[0]
        return func
    n_type_orlist = collapse_list("type_orlist")
    n_dim_orlist = collapse_list("dim_orlist")
    n_type_andlist = collapse_list("type_andlist")
    n_dim_andlist = collapse_list("dim_andlist")
    n_typelist = collapse_list("typelist")

    def n_dimlist(self, node):
        if(node.kids[0].type == "dimlist"):
            node.kids = tuple(node.kids[0].kids) + (self.annotateDim(node.kids[1]),)
        else:
            node.kids = (self.annotateDim(node.kids[0]),)
        return node 
  
    def n_varlist(self, node):
        if(node.kids[0].type == "varlist"):
            node.kids = tuple(node.kids[0].kids) + (self.processVar(node.kids[1]),)
        else:
            node.kids = (self.processVar(node.kids[0]),)
        return node 

    def processVar(self,node):
        if(node.type == '.'):
            return True
        elif(node.type == '*'):
            return False
        else:
            raise RuntimeError, "Unexpected character as dim var"

    def n_typenest(self,node):
        if(len(node.kids) > 1):
            if node.kids[2].type == "typenest":
                node.packed = node.kids[0].packed + [node.kids[1].type == "<"] + node.kids[2].packed
                node.kids = tuple(node.kids[0].kids) + tuple(node.kids[2].kids)
            else:
                node.packed = node.kids[0].packed + [node.kids[1].type == "<"]
                node.kids = tuple(node.kids[0].kids) + (node.kids[2],)
        else:
            node.packed = []
            node.kids = (node.kids[0],)
        return node
    
    def n_createtype(self, node):
        node.kids = [node.kids[0], None, None, None, None, False] #hasmissing, name, dims, subtypes, strict
        return node
  
    def n_createdim(self, node):
        node.kids = [node.kids[0], None, None, False] #hasmissing, name, strict
        return node

    def n_hasmissing(self, node):
        if(node.kids[0].type == "createtype"):
            node.kids[0].kids[1] = node.kids[1].type in ("?","$")
        elif(node.kids[0].type == "createdim"):
            node.kids[0].kids[1] = node.kids[1].type in ("?","$")
        else:
            raise RuntimeError, "Invalid AST!"
        return node.kids[0]
    
    def n_strict(self, node):
        if(node.kids[0].type == "createtype"):
            node.kids[0].kids[5] = True
        elif(node.kids[0].type == "createdim"):
            node.kids[0].kids[3] = True
        else:
            raise RuntimeError, "Invalid AST!"
        return node.kids[0]
   
    def n_dims(self, node):
        assert node.kids[0].type == "createtype", "Invalid AST!"
        if(len(node.kids) > 1):
            assert node.kids[1].type == "dimlist", "Invalid AST!"
            node.kids[0].kids[3] = node.kids[1]
        return node.kids[0]

    def n_subtypes(self, node):
        assert node.kids[0].type == "createtype", "Invalid AST!"
        assert node.kids[1].type == "typelist", "Invalid AST!"
        node.kids[0].kids[4] = node.kids[1]
        return node.kids[0]
    
    def n_namedim(self, node):
        if(node.kids[0].type == "createdim"):
            node.kids[0].kids[2] = node.kids[1].attr
        else:
            return node
        return node.kids[0]
    
    def n_typeelem(self, node):
        if(node.kids[0].type == "createtype"):
            node.kids[0].kids[2] = node.kids[1].attr
        else:
            return node
        return node.kids[0]
    
    def annotateDim(self,node):
        if not node.type == "createdim":
            return node
       
        depshape, has_missing, name, strict = node.kids
        if(name is None):
            return node

        if(depshape.type == "varlist"):
            dependent = depshape.kids
            while(dependent and dependent[-1] is False):
                dependent = dependent[:-1]
            dependent = tuple(dependent)
            shape = UNDEFINED
        elif(depshape.type == "integer"):
            dependent = tuple()
            shape = depshape.attr
        elif(depshape.type == "~"):
            dependent = "~"
            shape = UNDEFINED
        elif(depshape.type == "inherit"):
            dependent = None
            shape = None
        else:
            raise RuntimeError, "Invalid AST!"

        if(name in self.dim_annot):
            ahas_missing, adependent, ashape = self.dim_annot[name]
            if(not ashape is None):
                if(not shape is None and shape != ashape):
                    raise RuntimeError, "Similar named dim: " + name + " with different shape: " + str(shape) + ", " + str(ashape)
                shape = ashape

            if(not adependent is None):
                if(not dependent is None and dependent != adependent):
                    raise RuntimeError, "Similar named dim: " + name + "  with different dependent dims: " + str(dependent) + ", " + str(adependent)
                dependent = adependent

            if(not ahas_missing is None):
                if(not has_missing is None and has_missing != ahas_missing):
                    raise RuntimeError, "Similar named dim: " + name + "  with different has_missing flag: "  + str(has_missing) + ", " + str(ahas_missing)
                has_missing = ahas_missing
        
        self.dim_annot[name] = (has_missing, dependent, shape)
        return node#}}}

class TypeStringASTRewriterPass2(GenericASTRewriter):#{{{
    def process(self, tree):
        ntree = self.postorder(tree)
        return ntree
        
    def n_typenest(self,node):
        kids = list(node.kids)
        while(len(kids) > 1):
            right = kids.pop()
            assert right.type == "createtype", "Invalid AST!"
            assert kids[-1].type == "createtype", "Invalid AST!"
            kids[-1].kids[4]= AST(type="typelist",kids=(right,))
        
        return kids[0]#}}}

class TypeStringASTInterpreter(object):#{{{
    def __init__(self, dim_annot, refdims, env={}):
        self.env = env
        self.dim_annot = dim_annot
        self.dims = dict([(dim.name,dim) for dim in refdims])
    
    def visit(self, node, dimpos=0):
        name = 'n_' + str(node.type)
        if hasattr(self, name):
            func = getattr(self, name)
            return func(node, dimpos)
        else:
            raise RuntimeError, "Cannot find method to process: " + name

    def n_createtype(self, node, dimpos=0):
        assert node.type == "createtype", "Cannot create type from this specification!"
      
        typetoken, has_missing, name, dimlist, subtypelist, strict = node.kids

        if has_missing is None:
            has_missing = False

        if(typetoken.type == '?'):
            typename = "?"
        else:
            typename = typetoken.attr
        if(typename not in __typenames__):
            raise RuntimeError, "Unknown type name: " + str(typename)
        typecls = __typenames__[typename]
       
        
        kwargs = {}
        if has_missing:
            kwargs['has_missing'] = True

        if(not dimlist is None):
            
            dims = dimpaths.DimPath(*[self.visit(dimnode, dimpos + pos) for pos, dimnode in enumerate(dimlist.kids)])
            dimpos += len(dims)
            kwargs['dims'] = dims
        elif(issubclass(typecls,TypeArray)):
            dims = dimpaths.DimPath(dimensions.Dim(UNDEFINED,(True,) * dimpos))
            dimpos += 1
            kwargs['dims'] = dims
        
        if not subtypelist is None:       
            subtypes = tuple([self.visit(subtypenode, dimpos) for subtypenode in subtypelist.kids])
            kwargs['subtypes'] = subtypes

        if(issubclass(typecls,TypeTuple)):
            fieldnames = []
            for pos, subtypenode in enumerate(subtypelist.kids):
                if subtypenode.type == "createtype":
                    name = subtypenode.kids[2]
                elif subtypenode.type == "typeelem":
                    name = subtypenode.kids[1].attr
                else:
                    name = None
                if name is None:
                    fieldnames.append("f" + str(pos))
                else:
                    fieldnames.append(util.valid_name(name))
            kwargs['fieldnames'] = tuple(fieldnames)

        if 'dims' in kwargs and len(kwargs['dims']) > 1:
            assert typecls == TypeArray, "Multidimensional types only allowed with arrays"
            dims = kwargs['dims'][::-1]
            kwargs['dims'] = dims[:1]
            subtype = typecls(**kwargs)
            for pos in xrange(1,len(dims)):
                subtype = TypeArray(dims=dims[pos:(pos+1)], subtypes=(subtype,))
            return subtype
        else:
            return typecls(**kwargs)

    def n_typeelem(self, node, dimpos):
        return self.visit(node.kids[0],dimpos)

    def n_param(self, node,dimpos):
        if not node.kids[0].attr in self.env:
            raise RuntimeError, "Cannot find variable '" + str(node.kids[0].attr) + "' in given variables"
        return self.env[node.kids[0].attr]

    def n_createdim(self, node, dimpos):
        assert node.type == "createdim", "Cannot create dim from this specification!"
        dimnode, has_missing, name, strict = node.kids
        if has_missing is None:
            has_missing = False
        
        if(dimnode.type == "varlist"):
            dependent = tuple(dimnode.kids)
            shape = UNDEFINED
        elif(dimnode.type == "integer"):
            dependent = tuple()
            shape = dimnode.attr
        elif(dimnode.type == "~"):
            dependent = (True,) * dimpos
            shape = UNDEFINED
        elif(dimnode.type == "inherit"):
            dependent = None
            shape = None
        else:
            raise RuntimeError, "Invalid AST!"
      
        if(not name is None):
            assert name in self.dim_annot, "Unannotated named dim found!"
            if(not name in self.dims):
                has_missing,dependent,shape = self.dim_annot[name]
                if(dependent is None):
                    dependent = tuple()
                elif(dependent == "~"):
                    dependent = (True,) * dimpos
                    
                if(shape is None):
                    shape = UNDEFINED
                self.dims[name] = dimensions.Dim(shape,dependent,has_missing, name=name) 
                    
            dim = self.dims[name]
            if(len(dim.dependent) > dimpos):
                raise RuntimeError, "Dim: " + name + " has too many dependent dims: " + str(len(dim.dependent)) + " (max: " + str(dimpos) + ")"
            if dim.shape != shape and not shape is None and not shape == UNDEFINED and not dim.shape == UNDEFINED:
                raise RuntimeError, "Dim: " + name + " dimension unequal to known dim"

            return dim


        return dimensions.Dim(shape,dependent,has_missing, name=name) #}}}

def _createType(name, dimpos=0, refdims=[], env={}):
    scanner = TypeStringScanner()
    tokens = scanner.tokenize(name)

    parser = TypeStringParser()

    tree = parser.parse(tokens)
    
    #print tree
    rewriter1 = TypeStringASTRewriterPass1()
    tree, dim_annotation = rewriter1.process(tree)
    #print tree, dim_annotation
    rewriter2 = TypeStringASTRewriterPass2()
    tree = rewriter2.process(tree)
    #print "2: " + str(tree)
    return TypeStringASTInterpreter(dim_annotation, refdims, env).visit(tree, dimpos)


class TypeStringMatchASTInterpreter(object):
    def process(self, tree, dims, matchtype, env, dim_annotation):
        self.packdepth = 0
        self.error_message = ""
        self.env = env
        self.dim_annotations = dim_annotation
        res = self.n_typenest(tree,matchtype,dims)
        return (res, self.packdepth, self.error_message, self.env)
   
    def setError(self, message):
        self.error_message = message
    def unsetError(self):
        self.error_message = ""
    
    def visit(self, node, matchers):
        name = 'n_' + str(node.type)
        if hasattr(self, name):
            func = getattr(self, name)
            if isinstance(matchers,tuple):
                mres = []
                for matcher in matchers:
                    mres.append(func(node, matcher))
                res = any([x for x,y in mres])
                ms = sum([y for x,y in mres],())
            else:
                res, ms = func(node, matchers)
            return (res, ms)
        else:
            raise RuntimeError, "Cannot find method to process: " + name

    def paramvisit(self, node, dims):
        name = 'p_' + str(node.type)
        if hasattr(self, name):
            func = getattr(self, name)
            return func(node, dims)
        else:
            raise RuntimeError, "Cannot find method to process: " + name

    def n_typenest(self, node, matchtype, dims=None):
        assert node.type == "typenest", "Invalid AST!"
        packlist = node.packed + [False]
        for packed, kid in zip(packlist[::-1], node.kids[::-1]):
            if packed:
                res, dims = self.paramvisit(kid, dims) 
                if res is False:
                    return (False,())
            else:
                if self.packdepth > 0:
                    raise RuntimeError, "Cannot intermix packed and non-packed types (unpacked: <, packed: :)"

        dims = None
        matchtypes = matchtype
        for pos, (packed, matcher) in enumerate(zip(packlist, node.kids)):
            if packed:
                continue
            res, matchtypes = self.visit(matcher, matchtypes)      
            if res is False:
                return (False,())
        return (True,matchtypes)

    def subtype(self, matchtypes):
        if isinstance(matchtypes, tuple):
            res = []
            for matchtype in matchtypes:
                if hasattr(matchtype, 'subtypes') and len(matchtype.subtypes) == 1:
                    res.append(matchtype.subtypes[0])
            res = tuple(res)
        else:
            if hasattr(matchtypes, 'subtypes') and len(matchtypes.subtypes) == 1:
                res = matchtypes.subtypes
            else:
                res = ()
        return res

    def p_createtype(self, node, dims):
        if not dims:
            self.setError("Asked to pack dimension, but no dimension found")
            return False, None
        if  node.kids[0].attr == "array":
            raise RuntimeError, "Only array type allowed in packed type matcher"
        
        dimlist = node.kids[3]
        if any([d.type == "??" for d in dimlist.kids]):
            raise RuntimeError, "No variable dimension lists (??) allowed in packed type matcher"

        res, rdims = self.visit(dimlist, (dims[-len(dimlist):],))
        self.packdepth += len(dimlist)
        return (res, dims[:-len(dimlist)])
    
    def n_param(self, node, matchtype):
        name = node.kids[0].attr
        if name in self.env and  self.env[name] != matchtype:
            self.setError("Matches of multiple recurrences of variable: " + name + " are not equal")
            return False
        self.env[name] = matchtype
        return (True, self.subtype((matchtype,)))

    def n_type_orlist(self, node, matchtype):
        results = [self.visit(matcher, matchtype) for matcher in node.kids]
        res = any([x for x, m in results])
        ms = sum([m for x, m in results if x],())
        if res is True:
            self.unsetError()
        return (res, ms)
    
    def n_type_andlist(self, node, matchtype):
        results = [self.visit(matcher, matchtype) for matcher in node.kids]
        res = all([x for x, m in results])
        ms = sum([m for x, m in results],())
        return (res, ms)

    def n_tmatcher(self, node, matchtype):
        if node.kids[0].type == "^":
            res,ms = self.visit(node.kids[1], matchtype)
            res = not res
            if res is False:
                self.setError("Type " + str(matchtype) + " matches negative constraint")
            else:
                self.unsetError()
            return (res, ms)
        else:
            return self.visit(node.kids[0], matchtype)


    def n_createtype(self, node, matchtype):
        typetoken, has_missing, name, dimlist, subtypelist, strict = node.kids

        if(typetoken.type == '?'):
            return (True,())
        elif(typetoken.type == 'param'):
            res, ms = self.visit(typetoken, matchtype) 
        else:
            typename = typetoken.attr
            
            if(typename not in __typenames__):
                raise RuntimeError, "Unknown type name: " + str(typename)
            typecls = __typenames__[typename]
        
            if not isinstance(matchtype, typecls):
                self.setError("Type class mismatch for: " + str(matchtype) + ", expected: " + typecls.name)
                return (False,())

        
        if not has_missing is None and not matchtype.has_missing is has_missing:
            self.setError("Type: " + str(matchtype) + " has incorrect has_missing state")
            return (False,())
                
        if not dimlist is None:
            if not hasattr(matchtype, "dims"):
                self.setError("Required type with dimensions, but found: " + str(matchtype))
                return (False,())
            res,md = self.visit(dimlist, (matchtype.dims,))
            if res is False:
                return (False,())
        
        if not subtypelist is None:
            if not hasattr(matchtype, "subtypes"):
                self.setError("Required type with subtypes, but found: " + str(matchtype))
                return (False,())
            if subtypelist.type == "typelist":
                if not len(subtypelist.kids) == len(matchtype.subtypes):
                    self.setError("Incorect number of subtypes, expected: " + str(len(subtypelist.kids)) + " but found: " + str(len(matchtype.subtypes)))
                    return (False,())
                for subtype, kid in zip(matchtype.subtypes, subtypelist.kids):
                    res,ms = self.visit(kid, subtype)
                    if res is False:
                        return (False,())
            elif subtypelist.type == "createtype":
                if not len(matchtype.subtypes) == 1:
                    self.setError("Incorect number of subtypes, expected 1 but found: " + str(len(matchtype.subtypes)))
                    return False
                res,ms = self.visit(subtypelist, matchtype.subtypes)
                if res is False:
                    return (False,())
            else:
                raise RuntimeError, "Unexpected subtypelist in AST"

        return (True, self.subtype(matchtype))

    def n_typeelem(self, node, matchtype):
        return self.visit(node.kids[0], matchtype)

    def n_createdim(self, node, dim):
        if not isinstance(dim, dimensions.Dim):
            self.setError("Expected dimension, but found none")
            return (False,())

        dimnode, has_missing, name, strict = node.kids
        if not has_missing is None and not dim.has_missing is has_missing:
            self.setError("Dimension: " + str(dim) + " has incorrect has_missing state")
            return (False,())
        
        if not name is None:
            if isinstance(self.dim_annotations[name], dimensions.Dim):
                if not self.dim_annotations[name] == dim:
                    self.setError("Dimensions with same name: " + name + " in matcher do not match in type for " + str(self.dim_annotations[name]) + " and " + str(dim))
                    return (False,())
            self.dim_annotations[name] = dim

        if(dimnode.type == "varlist"):
            dependent = tuple(dimnode.kids)
            while(dependent and dependent[-1] is False):
                dependent = dependent[:-1]
            if not dim.dependent == dependent:
                self.setError("Dimenension " + str(dim) + " does have incorrect dependence struture")
                return (False,())
        elif(dimnode.type == "integer"):
            if not dim.shape == dimnode.attr:
                self.setError("Dimenension " + str(dim) + " does have incorrect shape (" + str(dim.shape) + " instead of " + str(dimnode.attr) + ")")
                return (False,())
        elif(dimnode.type == "~"):
            if not dim.dependent:
                self.setError("Dimenension " + str(dim) + " should be variable but is not")
                return (False,())
        else:
            raise RuntimeError, "Invalid AST!"

        return (True,())

    def n_dimlist(self, node, dims):
        return self.match(node.kids, dims)
    
    def n_nameddim(self, node, dims):
        return self.visit(node.kids[0], (dims,))
    
    def n_dim_orlist(self, node, dims):
        xdims = (dims,)
        results = [self.visit(matcher, xdims) for matcher in node.kids]
        res = any([x for x, m in results])
        ms = sum([m for x, m in results if x],())
        if res is True:
            self.unsetError()
        return (res, ms)
    
    def n_dim_andlist(self, node, dims):
        xdism = (dims,)
        results = [self.visit(matcher, xdims) for matcher in node.kids]
        res = all([x for x, m in results])
        ms = sum([m for x, m in results],())
        return (res, ms)

    def match(self, dim_matchers, dims):
        if not dim_matchers and not dims:
            return (True,())
        elif not dims:
            self.setError("Expected dimension, but found none")
            return (False,())
        elif not dim_matchers:
            self.setError("Expected no dimension, but found one")
            return (False,())

        if dim_matchers[0].type == "?":
            if len(dims) == 0:
                self.setError("Expected dimension, but found none")
                return (False,())
            return self.match(dim_matchers[1:], dims[1:])
        elif dim_matchers[0].type == '??':
            mres = [self.match(dim_matchers[1:],dims[pos:])  for pos in xrange(len(dims) + 1)]
            res = any([x for x,y in mres])
            if res is True:
                self.unsetError()
            return (res,())
        else:
            res, md = self.visit(dim_matchers[0], dims[0])
            if res is False:
                return (False,())
            return self.match(dim_matchers[1:], dims[1:])


def matchType(name, matchtype, env=None, dims=None):
    scanner = TypeStringScanner()
    tokens = scanner.tokenize(name)

    parser = TypeStringParser()

    tree = parser.parse(tokens)
    #print '1: ', tree  
    rewriter1 = TypeStringASTRewriterPass1()
    tree, dim_annotation = rewriter1.process(tree)
    #print '2: ', tree, dim_annotation

    if env is None:
        env = {}
    return TypeStringMatchASTInterpreter().process(tree, dims, matchtype, env, dim_annotation)

#### HELPER functions #########

def mostSpecializedTypesCls(typeclasses):
    """Takes list of typeclasses, and removes the 
    classes that also have a subclass of themself in the list"""
    basetypes = reduce(operator.__or__, 
                            [set(t.__mro__[1:]) for t in typeclasses])
    return [tcls for tcls in typeclasses  
                       if not tcls in basetypes]

def mostSpecializedTypes(typeobjs):
    """Takes list of typeobjects, and removes the 
    objects that also have a subclass of themself in the list"""
    basetypes = reduce(operator.__or__, 
                            [set(t.__class__.__mro__[1:]) for t in typeobjs])
    return [tobj for tobj in typeobjs  
                       if not tobj.__class__ in basetypes]


### sets ###
TypeAll = set(TypeAny.getDescendantTypes())
TypeNumbers = set(TypeNumber.getDescendantTypes())
TypeStrings = set(TypeString.getDescendantTypes()) - set([TypePickle])
TypeArrays = set(TypeArray.getDescendantTypes())
TypeIntegers = set(TypeInteger.getDescendantTypes())
TypeReals = set(TypeReal64.getDescendantTypes()) - TypeIntegers

if(platform.architecture()[0] == "32bit"):
    TypePlatformInt = TypeInt32
else:
    TypePlatformInt = TypeInt64
__typenames__["int"] = TypePlatformInt    
__typenames__['unknown'] = TypeUnknown



import urllib
import xml.dom.minidom
import csv

import wrapper
_delay_import_(globals(),"wrapper_py","Result")
_delay_import_(globals(),"..itypes","rtypes","dimensions")
_delay_import_(globals(),"slices")

def connect_biomart(url="http://www.biomart.org/biomart/martservice", query=None):
    if(query is None):
        return BiomartConnection(url)
    else:
        return BiomartRepresentor(url, query)
    return Connection(con)

class BiomartConnection(object):
    def __init__(self, engine):
        pass    

    def __getattr__(self, name):
        return self.tabledict[name]

    def _getAttributeNames(self):
        return self.tabledict.keys()
    
    def __repr__(self):
        res = "Database: " + str(self.engine.url) + "\n"
        tablenames = self.tabledict.keys()
        tablenames.sort()
        res += "Tables: " + ", ".join([str(tname) for tname in tablenames])
        return res

class BiomartMart(object):
    pass

class BiomartDataset(object):
    pass

class BiomartDatasetSection(object):
    pass

class BiomartAttribute(object):
    pass

class BiomartRepresentor(wrapper.SourceRepresentor):
    _select_indicator = None
    _select_executor = None
    
    def __init__(self, url, query):
        querydom = xml.dom.minidom.parseString(query)
        names = [str(node.getAttribute("name")) for node in 
                            querydom.getElementsByTagName("Attribute")]
        
        dataset_name = str(querydom.getElementsByTagName("Dataset")[0].getAttribute("name"))
        ndims = (dimensions.Dim("*", name=dataset_name.lower()),)


        subtypes = (rtypes.unknown,) * len(names)
        ntype = rtypes.TypeTuple(subtypes=subtypes, fieldnames=names)
        ntype = rtypes.TypeArray(subtypes=(ntype,), dims=ndims)
        tslices = (slices.Slice(dataset_name, ntype),)

        query_node = querydom.getElementsByTagName("Query")[0]
        query_node.setAttribute("header", "0")
        query_node.setAttribute("count", "0")
        assert query_node.getAttribute("formatter") == "TSV", \
                    "Only TSV format supported currently"


        self._encoded_query = url + "?" + urllib.urlencode([('query', str(querydom.toxml()))])


        all_slices = dict([(slice.id, slice) for slice in tslices])
        wrapper.SourceRepresentor.__init__(self, all_slices, tslices)

    def pyexec(self, executor):
        req = urllib.urlopen(self._encoded_query)
        reader = csv.reader(req, delimiter="\t")
        res = {}
        res[self._active_slices[0].id] = [row for row in reader]
        return Result(res)





from collections import defaultdict

class TopologicalSortable(object):
    after = set()
    before = set()


class topo_sorted(object):
    def __init__(self, objiter, return_index=False):
        self.objs = list(objiter)
        self.obj_idxs = range(len(self.objs))
        self.totalobj = len(self.objs)

        self.retcls = set()
        self.availcls = defaultdict(int)

        self.before_cls = defaultdict(set)
        self.return_index = return_index

        for obj in self.objs:
            if(self.availcls[obj.__class__] == 0):
                self.before_cls[obj.__class__].update(obj.before)
            self.availcls[obj.__class__] += 1

   
    def invalidate(self, cls):
        self.retcls.discard(cls)

    def _addobj(self, obj):
        if(len(obj.before & self.retcls) > 0):
            raise RuntimeError, "Cannot add " + str(obj) + \
                " as before constraint is violated by: " + \
                    str(obj.before & self.retcls)

        if(self.availcls[obj.__class__] == 0):
            self.before_cls[obj.__class__].add(obj.before)
        
        self.availcls[obj.__class__] += 1
    
    def __contains__(self, obj):
        if(issubclass(obj, TopologicalSortable)):
            return self.availcls[obj] > 0
        else:
            return obj in self.objs

    def append(self, obj):
        self._addobj(obj)
        self.objs.append(obj)
        self.obj_idxs.append(self.totalobj)
        self.totalobj += 1
        return self.totalobj

    def prepend(self, obj):
        self._addobj(obj)
        self.objs.insert(0, obj)
        self.obj_idxs.insert(0, self.totalobj)
        self.totalobj += 1
        return self.totalobj
    
    def __iter__(self):
        return self

    def next(self):
        for pos, objcls in enumerate(self.objs):
            #after constraints fullfilled?
            if(not objcls.after.issubset(self.retcls)):
                continue

            #are there objects that want to run before me?
            if(sum(self.availcls[before] 
                    for before in self.before_cls[objcls])):
                continue
            idx = self.obj_idxs[pos]
            del self.objs[pos]
            del self.obj_idxs[pos]

            self.availcls[objcls] -= 1
            self.retcls.add(objcls)
            if(self.return_index):
                return idx
            else:
                return objcls
        
        if(not self.objs):
            raise StopIteration
        
        raise RuntimeError, "Cycle in topological sort: " + \
                                    "cannot return next object"



  



def load_arrayexpress():
    if(os.path.exists('array_express.data')):
        print "Loading cached array express data..."
        r = util.load_rep('array_express.data')
    else:
        print "Loading array express data..."
        aexpress = wrapper_tsv.read_tsv('/home/marc/Databases/listdl.tab')
        
        print "Grouping array express data on gene identifier and condition..."
        aexpress = aexpress.group_by(_.gene_identifier, _.factor_value, keep={1:("experimental_factor",)}).get(_.gene_identifier, _.factor_value, _.experimental_factor, _.expression).copy()
        ae_down = numpy.array(map(list, aexpress.expression[_ == "DOWN"].count()()),dtype=int)
        ae_up = numpy.array(map(list, aexpress.expression[_ == "UP"].count()()),dtype=int)
        ae_count = ae_up - ae_down
        ae_genenames = dict([(name, gpos) for gpos, name in enumerate(aexpress.gene_identifier())])
        
        #normalize per condition
        ae_count = ae_count / ae_count.sum(axis=0)

        eset = aexpress.experimental_factor.set()()
        ecounts = []
        ename = []
        for name in eset:
            eindex = aexpress.experimental_factor.get(pos()/"pos",_)[_.experimental_factor == name].pos()
            ecounts.append(ae_count[:,eindex])
            ename.append(name)
         
        r = (ecounts, ae_genenames, ename)
        util.save_rep(r, 'array_express.data')
    return r


def load_chipchip(gn_to_orf):
    if(os.path.exists('chipchip.data')):
        print "Loading cached chipchip data..."
        res = util.load_rep('chipchip.data')
    else:
        print "Parsing and combining chipchip data..."
        chipchip = wrapper_chipchip.read_chipchip('/home/marc/Databases/chip-chip/');

        print "Loading chipchip data and matching it to gene names..."
        #get orf code for chipchip regulator name by matching with gn list. Filter out non matched regulators, return first match (should all be equal)
        #chipchip_orf = gn_to_orf[{_.gene_name:chipchip.regulator}].hstack(chipchip)[_.gene_name]
        chipchip_orf = chipchip.match(gn_to_orf,"regulator","gene_name", group=True)[_.orf_code.count() > 0].get(_.orf_code[0], _.gene_name[0],  "~")

        #flatten chipchip list, rename orf_code field to orf_reg
        cc = chipchip_orf.flat()/{"orf_code":"orf_reg"}

        #filter out orfs not available in sgd orfs list
        cc = cc.match(gn_to_orf, "orfs", "gene_name", group=True)[_.orf_code.count() > 0].get(_.orf_code[0]/"orfs", "~").copy()
        
        yt = load_yeastract(gn_to_orf)
        cc = (cc.get(_.orf_reg, _.orfs, _.max_pval, _.conservation).tuple().array() + yt.join(rep([(-1.0, -1.0)])).tuple().array()).E.A.copy()

        ccdata = cc.get(_.max_pval, _.conservation).tuple()()
        ccnames = cc.get(_.orf_reg, _.orfs)()
        res = (ccdata, ccnames)
        util.save_rep(res, 'chipchip.data')
    return res


def load_phospho(gn_to_orf):
    pp = wrapper_phospho.read_phospho('/home/marc/Databases/phospho/')
    s = numpy.std(pp.score().view(numpy.ndarray))
    pp = pp.match(gn_to_orf, "kinase", "gene_name").get(_.orf_code/"kinase_orf", _.orf, (_.score/s)/"score")
    pp = pp.match(gn_to_orf, "orf", "gene_name").get(_.kinase_orf, _.orf_code/"target_orf", _.score).copy()
    return pp

def load_yeastract(gn_to_orf):
    yt = wrapper_tsv.read_tsv('/home/marc/Databases/yeastract/regul.tsv')/("tf_name", "regul_name")
    yt = yt.map(lambda x:x.upper(),otype="string[]")
    yt = yt.match(gn_to_orf, "tf_name", "gene_name").get(_.orf_code/"tf_orf", _.regul_name)
    yt = yt.match(gn_to_orf, "regul_name", "gene_name").get(_.tf_orf, _.orf_code/"regul_orf").copy()
    return yt

def add_protprot_edges(network, gn_to_orf):
    if(os.path.exists('protprot.data')):
        res = util.load_rep('protprot.data')
    else:
        print "Loading yeast protein-protein data..."
        yeast_bc = wrapper_psimi.read_psimi('yeast.xml').copy()

        print "Mapping interactor names to  gene orf names..."
        interactors = yeast_bc.interactors.A.match(gn_to_orf, "short_name", "gene_name").get(_.id, _.orf_code).copy()

        print "Unpacking interactions and filtering genetic interference interactions..."
        interactions = yeast_bc.interactions.get(_.biogrid_type, _.participants.get(_.interactor_id, _.role), _.experiments.flat()).copy()
        experiments = yeast_bc.experiments[_.method_name != "genetic interference"].id
        interactions = interactions.match(experiments, "experiments", "id").L.copy()

        print "Unpacking baits and preys..."
        interactions = interactions.get(_.biogrid_type, _[_.role == "bait"].interactor_id[0]/"bait", _[_.role == "prey"].interactor_id/"prey").flat("prey").copy()
        
        print "Matching interactions with interactors..."
        interactions = interactions.match(interactors/{"orf_code": "bait_orf"}, "bait", "id").match(interactors/{"orf_code":"prey_orf"}, "prey", "id").get(_.biogrid_type, _.bait_orf, _.prey_orf).copy()

        print "Grouping data on interaction pair and measurement type and count different measurement types per interaction pair"
        interactions = interactions.join(rep([1.0])/"score").copy()
        
        pp = load_phospho(gn_to_orf)
        interactions = (interactions.get("bait_orf", "prey_orf", "biogrid_type", "score").tuple().array() + interactions.get("prey_orf", "bait_orf", "biogrid_type", "score").tuple().array())
        interactions = interactions + pp.join(rep(["kinase"])/"biogrid_type").get(_.kinase_orf, _.target_orf, _.biogrid_type, _.score).tuple().array() 

        res = interactions.E.A.get(_.get(_.bait_orf, _.prey_orf).tuple()/"pair", _.biogrid_type, _.score).group_by(_.pair, _.biogrid_type)
        res = res.get(_.pair.A, _.biogrid_type, _.score.sum()).copy()

        icount = res.group_by(_.bait_orf).get(_.bait_orf, _.prey_orf.set().E.count()/"interaction_count")
        res = res.match(icount, "bait_orf", "bait_orf")/{"interaction_count":"bait_count"}
        res = res.match(icount/{"bait_orf":"prey_orf"}, "prey_orf", "prey_orf")/{"interaction_count":"prey_count"}
        res = res.get(_.bait_orf, _.prey_orf, _.biogrid_type, _.score, _.bait_count, _.prey_count).copy()()
        #network.addEdges("protprot", ccnames[1], ccnames[0], res, cctypes)
        res[4].shape = res[4].shape + (1,)
        res[5].shape = res[5].shape + (1,)

        dres = numpy.concatenate((numpy.array(map(list,res[3]), dtype="uint16"), res[4], res[5]), axis=1)        
        #filter = ((dres[:,-1] < 10) & (dres[:,-2] < 10)) | (dres[:,-3] != 0)
        res = (res[0], res[1], res[2], dres[:,:-2])
        #res = (res[0][filter], res[1][filter], res[2], dres[filter,:-2])

        util.save_rep(res, 'protprot.data')
    network.addEdges("protprot", res[0], res[1], res[3], list(res[2]), [0] * len(res[2]))
#    network.addEdges("protprot", nres[0], nres[1], nres[3], list(nres[2]) + ['bait_count', 'prey_count'])
    return res
    

def load_gntoorf():
    print "Loading gene orf table..."
    gn = wrapper_tsv.read_tsv('/home/marc/Databases/SGD_features.tab', 
            fieldnames=('sgdid', 'feat_type', 'feat_qual', 'feat_name', 'gene_name', 'alias', 
                        'parent_feat_name', 'sgdid2', 'chromosome', 'start', 'stop', 'strand' ,
                        'genetic_pos', 'coordinate_version', 'sequence_version', 'description'));    
    
    gn = gn[_.feat_type == "ORF"]/{'feat_name':'orf_code', 'alias':'alt_gene_names'}
    #Split alt_gene names
    print "Loading gene names and creating name translation table..."
    gn = gn.get(_.gene_name, _.alt_gene_names.map(lambda x: str.split(x,'|'), otype="[symonyms:~](bytes[])").E, "~").copy()

    #create a list from gene names, alt genenames and orf code to of code
    gn_to_orf = (gn.get(_.gene_name, _.orf_code).tuple().array() + 
                 gn.get(_.alt_gene_names, _.orf_code).flat().tuple().array() + 
                 gn.get(_.orf_code, _.orf_code).tuple().array()).E.A.copy()
    
    gn_to_orf = gn_to_orf[_.gene_name != ""].group_by(_.gene_name).get(_.gene_name, _.orf_code[0]).copy()

    return (gn, gn_to_orf)

def addprotein_dna_edges(network, gn_to_orf):
    if(os.path.exists('protdna.data')):
        print "Loading cached protein dna edges..."
        res, ccnames, cctypes = util.load_rep('protdna.data')
    else:
        ae_counts, ae_genenames, ae_expfactor = load_arrayexpress()
        ccdata, ccnames = load_chipchip(gn_to_orf)

        #add chipchip data, based on out orfs, in orfs, and feature data (pval, conservation)
        print "Adding protein-dna edges..."
        set_ccdata = util.darray(list(set(ccdata)),object,1)
        cctypes = []
        res = numpy.zeros((len(ccdata), len(set_ccdata) + len(ae_expfactor)),dtype=float)
        for tpos in xrange(len(set_ccdata)):
            cctype = set_ccdata[tpos:tpos + 1]
            res[numpy.equal(ccdata,cctype),len(cctypes)] = 1
            cctypes.append("({0:.3f}, {1})".format(cctype[0][0],int(cctype[0][1])))

        arraypos = len(set_ccdata)

        for cpos, (regname, targetname) in enumerate(zip(*ccnames)):
            if(regname in ae_genenames and targetname in ae_genenames):
                posl = ae_genenames[regname]
                posr = ae_genenames[targetname]
                for colpos, ae_count in enumerate(ae_counts):
                    corr = numpy.corrcoef(ae_count[posl,:], ae_count[posr,:])[0,1]
                    if(not numpy.isnan(corr)):
                        res[cpos,arraypos + colpos] = numpy.abs(corr) ** 2
        cctypes.extend(ae_expfactor)
        xsave = (res, ccnames, cctypes)
        util.save_rep(xsave, 'protdna.data')
    network.addEdges("protdna", ccnames[0], ccnames[1], res, cctypes, [0] * len(cctypes))


def load_network():
    #reading in the data
    print "Loading knockout experiment table..."
    experiment_list, hughes_data, control_data = wrapper_hughes.read_hughes('/home/marc/Databases/hughes');

    gn, gn_to_orf = load_gntoorf()
    print "Loading hughes control data and mapping it to gene names and removing double orfs..."
    #create measurment nodes: determine which orfs were measured in the hughes dataset
    cgenes = control_data.match(gn, "gene_official", "orf_code")
    #remove double occuring orf
    cgenes = cgenes.group_by(_.orf_code)[_.gene_official.count() == 1].flat("gene_official").copy()

    print "Creating network nodes..."
    #create nodes
    node_ids = gn.get(_.orf_code/"nodeid", _.gene_name, _.description, pos("gene_name")/"node_pos").copy()
    #create network based on node ids
    network = Network(node_ids.nodeid())

    print "Adding node attributes..."
    network.addNodeAttributes("description", node_ids.description())
    network.addNodeAttributes("gene_name", node_ids.gene_name())

    addprotein_dna_edges(network, gn_to_orf)
    add_protprot_edges(network, gn_to_orf)
   
    print "Adding measurement edges..."
    #add gene node to measurement node edges
    cmean = [numpy.mean(row[~numpy.isnan(row)]) for row in cgenes.intensity()]
    cstd = [numpy.std(row[~numpy.isnan(row)]) ** 2 for row in cgenes.ratio()]
    network.addEdges("measurement", cgenes.orf_code(), cgenes.orf_code(), numpy.array([cmean,  cstd]).T, ["intensity", "std"], [0,0])

    #TARGET DATA

    kdata = (wrapper_tsv.read_tsv('/home/marc/Databases/yeast_knockouts/p_knockouts.txt')/('knockout_name', 'target_name','p_value'))
    kdata = kdata.match(node_ids, "target_name", "nodeid").get(_.nodeid/"target_orf", _.knockout_name, _.p_value, _.node_pos/"gene_pos")
    kdata = kdata.match(node_ids, "knockout_name", "nodeid").get(_.nodeid/"knockout_orf", _.target_orf, _.p_value, _.gene_pos).copy()
    kdata = kdata.group_by(_.target_orf, _.knockout_orf, keep={(0,):("gene_pos",), (0,1):("p_value",)}).copy()

    target_dictn = {}
    for node,pvals in zip(kdata.knockout_orf(), kdata.p_value().T):
        target_dictn[node] = 1 - pvals

    #determine position of gene names in nodeids list(network), by matching them with positioned node list, 
    #sorting result on gene pos (thus using order in pvals), and returning node pos list
    target_dictn["__sel__"] = kdata.gene_pos()


    #filter experiment list on knockouts that are not part of the titration series
    print "Loading experiment data and use it to translate knockout names..."
    experiment_list = experiment_list[_.repeated.map(lambda x: not "titration" in x, otype="bool")]

    #Create a position for each knockout (to use as index after joining). Match knockouts in hughes data with hughes experiment list data to get orf codes. 
    #knockouts = hughes_data.knockouts.row_dict()[experiment_list.experiment_name].hstack(experiment_list.orf_code)
    knockouts = hughes_data.knockouts.get(pos()/"pos", _).match(experiment_list, "knockouts", "experiment_name")
    
    #Translate orf codes to upper case, keep only those rows for which the orf_code is unique
    #knockouts = knockouts.get(_.orf_code.map(str.upper, otype="bytes[]"), "~").over(group=_.orf_code, do=_[_.experiment_name.count() == 1])
    knockouts = knockouts.get(_.orf_code.map(str.upper, otype="bytes[]"), "~").group_by(_.orf_code)[_.experiment_name.count() == 1].flat()

    #match knockouts with genelist orf codes (to filter out non-existing), keep position and orf_code
    #knockouts = knockouts[_.orf_code.within(gn.orf_code)]
    print "Matching knockouts to orf names..."
    knockouts = knockouts.match(gn, "orf_code", "orf_code").get(_.pos, _.orf_code, _.experiment_name).copy()

    #get gene names from each measured transcript in hughes data, note position of gene name
    #then match with genenames, keep gene official name (orf code) and position

    print "Loading knockout pvalues and creating target data..."
    genes = hughes_data.match(node_ids, "gene_official", "nodeid")[ismissing(_.pval).sum() == 0].copy()
    pvals = genes.pval()
    
    #replace pvals == 1 with NaN
    sel_missing = numpy.equal(pvals,1)
    pvals[sel_missing] = numpy.nan

    target_dict = {}
    for node,knockout_pos in zip(knockouts.orf_code(), knockouts.pos()):
        target_dict[node] = 1 - pvals[:, knockout_pos]

    #determine position of gene names in nodeids list(network), by matching them with positioned node list, 
    #sorting result on gene pos (thus using order in pvals), and returning node pos list
    target_dict["__sel__"] = genes.node_pos()
    return (network, target_dict, target_dictn)





"""
Parser for PSI-MI files.
Specification overview: http://dip.doe-mbi.ucla.edu/psimi/MIF254.xsd
"""

import re
from itertools import chain
import sys, string
import xml.sax.handler
import xml.sax
from collections import deque,defaultdict, namedtuple
import xml.parsers
import xml.parsers.expat
import numpy

from ..utils.multi_visitor import DirectVisitorFactory, NF_ELSE
from ..utils.missing import Missing
import wrapper
from ..itypes import rtypes
from .. import ops
from ..constants import *
import python
from ..utils import nested_array,util

def read_psimi(filename):
    return PSIMIRepresentor(filename).Fields().Copy()

Interaction = namedtuple('Interaction', ('id', 'type', 'participants', 'experiments', 'biogrid_type', 'author_confidence', 'intact_confidence'))
interaction_type= "[interactions:*]<(id=int32, type=string?, " + \
                    "participants=[iinteractors:~]<(id=int32, interactor_id=int32, role=string[]?), " + \
                    "experiments=[iexperiemnts:~]<int32, biogrid_type=string[]?, author_confidence=string[]?, intact_confidence=real32?" + \
                    ")"
Participant = namedtuple('Participant', ('id', 'interactor_id', 'role'))

Interactor = namedtuple('Interactor', ('id', 'short_name', 'refseq_id', 'type_id'))
interactor_type = "[interactors:*]<(id=int32, short_name=string[], refseq_id=string[]?, type_id=string[]?)"

Experiment = namedtuple('Experiment', ('id', 'method_id', 'method_name', 'pubmed_id'))
experiment_type = "[experiments:*]<(id=int32, method_id=string[], method_name=string[], pubmed_id=string[])"

Organism = namedtuple('Organism', ('id', 'short_label', 'full_name'))

class PSIMIRepresentor(wrapper.SourceRepresentor):
    def __init__(self, filename):
        xtype = "(experiment=" + experiment_type + ", interactor=" + interactor_type + ", interaction=" + interaction_type + ")"
        xtype = rtypes.createType(xtype)
        slice = PSIMIOp(filename,xtype,"data")
        self._initialize((slice,))



class PSIMIOp(ops.ExtendOp):
    __slots__ = ['filename']
    def __init__(self, filename, rtype, name="data"):
        self.filename = filename
        ops.ExtendOp.__init__(self, name=name, rtype=rtype)

    def py_exec(self):
        file = open(self.filename)
        psimi_handler = PSIMIParser()
        parser = xml.parsers.expat.ParserCreate()
        parser.StartElementHandler = psimi_handler.startElement
        parser.EndElementHandler = psimi_handler.endElement
        parser.CharacterDataHandler = psimi_handler.characters
        #parser.ExternalEntityRefHandler = psimi_handler.resolveEntity
        parser.UseForeignDTD(True)
        parser.buffer_text = True
        parser.ParseFile(file)

        file.close()
        exps = [Experiment(**exp) for exp in psimi_handler.experiments.values()]
        interactors = [Interactor(**inter) for inter in psimi_handler.interactors.values()]
        interactions = []
        for interaction in psimi_handler.interactions.values():
            interaction['participants'] = [Participant(**par) for par in interaction['participants']]
            interactions.append(Interaction(**interaction))

        data = (exps, interactors, interactions)
        ndata = nested_array.NestedArray(data,self.type)
        return python.ResultOp.from_slice(ndata,self)
        


"""
Handler class for actually processing the file
"""    
class PSIMIParser(DirectVisitorFactory(prefixes=("start", "end"), flags=NF_ELSE), xml.sax.handler.ContentHandler):

   def __init__(self):
      self.state = deque()
      self.interactions = {}
      self.cur_interaction = None
      self.interactors = {}
      self.cur_interactor = None
      self.experiments = {}
      self.cur_experiment = None
      self.organisms = {}
      self.cur_organism = None

      self.int_participants = []
      self.cur_participant = None
      self.int_experiments = []


      self.cur_attribute = ""
      self.cur_confidence_type = ""

   def startelse(self, name, attrs):
      pass

   def startinteraction(self, name, attrs):
      self.cur_interaction = {'id': int(attrs['id']), 'author_confidence':Missing, 'intact_confidence':numpy.nan, 'biogrid_type':Missing, 'type':Missing}
      self.int_experiments = []
      if('negative' in attrs):
          #print "Found negative interaction!!"
          raise Exception, 'negative interaction encountered, implement me!'
   
   def startparticipantList(self, name, attrs):
       self.int_participants = []

   def startparticipant(self, name, attrs):
       self.cur_participant = {'id':int(attrs['id']), 'role':Missing}

   def endparticipant(self, name):
       self.int_participants.append(self.cur_participant)
       
   def endparticipantList(self, name):
       self.cur_interaction['participants'] = self.int_participants
  
   def endinteractorRef(self, name):
       if(self.state[-2] == "participant"):
           self.cur_participant["interactor_id"] = int(self.last_nodedata)
  
   def endexperimentRef(self, name):
       self.int_experiments.append(int(self.last_nodedata))
   
   def endexperimentDescription(self, name):
       self.experiments[self.cur_experiment['id']] = self.cur_experiment
       self.cur_experiment = None

   
   def endinteraction(self, name):
       self.cur_interaction['experiments'] = self.int_experiments
       self.interactions[self.cur_interaction['id']] = self.cur_interaction
       self.cur_interaction = None
  
   def startinteractor(self, name, attrs):
       self.cur_interactor = {'id':int(attrs['id']), 'refseq_id':Missing,
                               'type_id':Missing}
       if(self.state[-2] == "participant"):
            self.cur_participant["interactor_id"] = int(attrs['id'])
   
  
   def endinteractor(self, name):
       self.interactors[self.cur_interactor['id']] = self.cur_interactor
       self.cur_interactor = None
   

   def startexperimentDescription(self, name, attrs):
       self.cur_experiment = {'id':int(attrs['id'])}
       if(self.state[-3] == "interaction"):
           self.int_experiments.append(int(attrs['id']))
        

   def startprimaryRef(self, name, attrs):
        if(attrs['id'] == 'unknown'):
            return
        if(attrs['db'] == 'psi-mi'):
            if(self.state[-3] == 'interactionDetectionMethod'):
                if(self.state[-4] == 'experimentDescription'):
                    self.cur_experiment['method_id'] = str(attrs['id'])
            elif(self.state[-3] == "interactorType"):
                self.cur_interactor['type_id'] = str(attrs['id'])
        elif(attrs['db'].lower() == 'pubmed'):
            if(self.state[-4] == 'experimentDescription' and self.state[-3] == "bibref"):
                self.cur_experiment['pubmed_id'] = str(attrs['id'])
        elif(attrs['db'] == 'refseq'):
            if(self.state[-3] == 'interactor'):
                self.cur_interactor['refseq_id'] = str(attrs['id'])

   startsecondaryRef = startprimaryRef
  
   def endshortLabel(self, name):
       if(self.state[-3] == "interactor"):
           self.cur_interactor['short_name'] = self.last_nodedata
       elif(self.state[-3] == "interactionType"):
           self.cur_interaction['type'] = self.last_nodedata
       elif(self.state[-3] == "experimentalRole"):
           self.cur_participant["role"] = self.last_nodedata
       elif(self.state[-4] == "confidence"):
           self.cur_confidence_type = self.last_nodedata
       elif(self.state[-3] == "interactionDetectionMethod"):
           self.cur_experiment['method_name'] = self.last_nodedata
   

   def endvalue(self, name):
       if(self.state[-2] == "confidence"):
           if(self.cur_confidence_type == "author-confidence"):
               self.cur_interaction["author_confidence"] = self.last_nodedata
           elif(self.cur_confidence_type == "intact confidence"):
               self.cur_interaction["intact_confidence"] = float(self.last_nodedata)
               
   def startattribute(self, name, attr):
       self.cur_attribute = attr["name"]
   
   def endattribute(self, name):
       if(self.cur_attribute == "BioGRID Evidence Code"):
           self.cur_interaction['biogrid_type'] = self.last_nodedata

   def endelse(self, name):
       pass

   def startElement(self, name, attrs):                        
       self.state.append(name)
       self.start(name, attrs)

   def endElement(self, name):
       self.end(name)
       self.state.pop()

   def characters(self, content):                 
       self.last_nodedata = content
   
   def error(self, exception):
        print "ERROR:" + str(exception)

   def warning(self, exception):
        print "WARNING: " + str(exception)

   def fatalError(self, exception):
        raise exception




_delay_import_(globals(),"..representor")
_delay_import_(globals(),"util")

#after receipt by Ferdinand Jamitzky on activestate.com: http://code.activestate.com/recipes/384122-infix-operators/
class Infix:
    def __init__(self, function, params = (), kwargs={}):
        self.function = function
        self.kwargs = kwargs
        self.params = params

    def __ror__(self, other):
        return Infix(lambda x, y, z, self=self, other=other: self.function(other, x, *y, **z), self.params, self.kwargs)

    def __or__(self, other):
        return self.function(other, self.params, self.kwargs)

    def __call__(self, *params, **kwargs):
        kwargs.update(self.kwargs)
        params = self.params + params
        
        if(params and isinstance(params[0],representor.Representor)):
            return self.function(*params, **kwargs)
        else:
            return Infix(self.function,params,kwargs)
            



import operator
from itertools import chain
from collections import defaultdict
import numpy
import sys
import cPickle
import time

import wrapper
from ..constants import *
from ..utils.multi_visitor import VisitorFactory, DirectVisitorFactory, NF_ERROR, NF_ELSE
from ..itypes import rtypes, dimpaths
from ..passes import manager, create_graph, serialize_exec
from .. import ops

from ..utils.multi_visitor import VisitorFactory, DirectVisitorFactory, NF_ELSE

_delay_import_(globals(),"..representor")
_delay_import_(globals(),"..utils","util","cutils","nested_array","context")
_delay_import_(globals(),"..itypes","detector","type_attribute_freeze","convertors","dimensions")
_delay_import_(globals(),"..repops_slice")
_delay_import_(globals(),"..repops_dim")
_delay_import_(globals(),"..repops_funcs")
_delay_import_(globals(),"..utils.missing","Missing")


class PyRepresentor(wrapper.SourceRepresentor):
    def __init__(self, slices):
        self._initialize(slices)

def Rep(data=None, dtype=None, unpack=True, name=None, allow_convert=False):
    """Packs python data structures into a :py:class:`ibidas.representor.Representor` object.
        
       :param data: any python object
       :param dtype: type of ``data``. See :py:func:`createType` for allowed string formats
                     (default=autodetect).
       :type dtype: :py:class:`str` or :py:class:`ibidas.itypes.rtypes.Type`
       :param unpack: determines if data object is unpacked. 
                      For example, unpacking an array means that subsequent operations are performed 
                      on the elements instead of on the array structure
       :param name: Name of the main slice, representing the data (default='data').
       :type name: :py:class:`str`, lower case
      
       Examples:
            Let the python data that needs to be represented be a simple table (collection of tuples):
            
            >>> data = [('gene1',0.5),('gene2',0.3),('gene100',0.9)]

            The, to load using type autodetection, use:

            >>> r = Rep(data)
            Slices: f0       f1     
            Types:  bytes[7] real64 
            Dims:   d1:3     .   
            
            Specifying type directly:

            >>> r = Rep(data,"[genes]<(name:string, value:real64)")
            Slices: name    value
            Types:  string  real64
            Dims:   genes:* .

            Effect of setting unpack to False:

            >>> r = Rep(data,"[genes]<(name:string, value:real64)", unpack=False)
            Slices: data                                  
            Types:  [genes:*]:(name=string, value=real64) 
            Dims: 
            
            Specifying root slice name:

            >>> r = Rep(data,"[genes]<(name:string, value:real64)", unpack=False, name="gene_table")
            Slices: gene_table                                  
            Types:  [genes:*]:(name=string, value=real64) 
            Dims: 

    """
    if(not dtype is None):
        if(isinstance(dtype,basestring)):
            dtype = rtypes.createType(dtype)
    else:
        det = detector.Detector()
        det.process(data)
        dtype = det.getType()
        if(dtype == rtypes.unknown):
            dtype = rtypes.TypeAny(True)

    if(name is None):
        res = util.find_names([data])
        if res:
            name = res[0]
        else:            
            name = "data"

    data_slice = ops.DataOp(data,name=name,rtype=dtype)
    
    while(unpack and data_slice.type.__class__ is rtypes.TypeArray):
        data_slice = ops.UnpackArrayOp(data_slice)

    res = wrapper.SourceRepresentor()
    res._initialize((data_slice,)) 

    if(unpack and isinstance(res._getType(), rtypes.TypeTuple)):
        res = repops_slice.UnpackTuple(res)
   
    if allow_convert:
        res = res.Detect()
    return res


class ResultOp(ops.DataOp):
    __slots__ = ["source"]
    def __init__(self, data=None, name=None, rtype=rtypes.unknown, dims=dimpaths.DimPath(), bookmarks=set()):
        self.name = name
        self.type = rtype
        self.dims = dims
        self.data = data
        self.bookmarks = bookmarks

    @classmethod
    def from_slice(cls,data,slice):
        return cls(data,slice.name, slice.type,slice.dims,slice.bookmarks)

    def modify(self,data=NOVAL, name=NOVAL, rtype=NOVAL, dims=NOVAL, bookmarks=NOVAL):
        if(name is NOVAL):
            name = self.name
        if(rtype is NOVAL):
            rtype = self.type
        if(dims is NOVAL):
            dims = self.dims
        if(data is NOVAL):
            data = self.data
        if(bookmarks is NOVAL):
            bookmarks = self.bookmarks

        return ResultOp(data, name, rtype, dims, bookmarks)

    def setSource(self,source):
        self.source = source
        self.bookmarks = source.bookmarks
        self.name = source.name
        self.dims = source.dims

    def __repr__(self):
        res = "\nName: " + str(self.name) + ", Type:   " + str(self.type) + ", Dims: " + str(self.dims) 
        try:
            r = str(self.data)
            r.decode('utf-8')
        except UnicodeDecodeError:
            return res
        res += '\n' + r
        return res

class PyExec(VisitorFactory(prefixes=("visit","unpackCast"), flags=NF_ELSE), 
             DirectVisitorFactory(prefixes=("cast",),flags=NF_ERROR), manager.Pass):

    after = set([create_graph.CreateGraph, serialize_exec.SerializeExec])

    @classmethod
    def run(cls, query, run_manager,debug_mode=False):
        self = cls()
        self.graph = run_manager.pass_results[create_graph.CreateGraph]
        commands = run_manager.pass_results[serialize_exec.SerializeExec]

        arguments = numpy.zeros((len(commands),),dtype=object)
        use_counters = numpy.zeros((len(commands),),dtype=int)
        param_idxs = self.graph.na["param_idxs"]
        param_usecount = self.graph.na["param_usecount"]

        for command_id, command in enumerate(commands):
            params = param_idxs[command]
          
            param_kwds = dict()
            for name,param_ids in params.iteritems():
                param_kwds[name] = arguments[param_ids]
                use_counters[param_ids] -= 1
                
            if(debug_mode):
                start = time.time()
            try:
                res = self.visit(command, **param_kwds)
            except Exception, e:
                exc_info = sys.exc_info()
                if(debug_mode):
                    try:
                        from ..passes.cytoscape_vis import DebugVisualizer
                        DebugVisualizer.run(query,run_manager)
                    except Exception:
                        raise
                raise exc_info[1], None, exc_info[2]
            if(debug_mode):
                rtime = time.time() - start
                self.graph.na["output"][command] = str(res)
                self.graph.na["time"][command] = rtime
                self.graph.na["time_readable"][command] = util.format_runtime(rtime)

            arguments[command_id] = res
            use_counters[command_id] = param_usecount[command]
            arguments[use_counters == 0] = None
        return res

    def visitelse(self,node, **kwargs):
        raise RuntimeError, "Unknown node type encountered: " + node.__class__.__name__ + " with  " + str(kwargs)

    def visitExtendOp(self,node, *args, **kwargs):
        return node.py_exec(*args, **kwargs)

    def visitDataOp(self,node):
        if(isinstance(node.data, nested_array.NestedArray)):
            ndata = node.data.copy()
        else:
            ndata = nested_array.NestedArray(node.data,node.type)
        return ResultOp.from_slice(ndata,node)

    def visitCastOp(self,node,slice):
        ndata = self.cast(node.cast_name,node,slice)
        return slice.modify(data=ndata,rtype=node.type)

    def visitChangeNameOp(self,node,slice):
        return slice.modify(name = node.name)

    def visitChangeDimPathOp(self,node,slice):
        return slice.modify(dims=node.dims)

    def visitChangeBookmarkOp(self,node,slice):
        return slice.modify(bookmarks=node.bookmarks)
    
    def visitChangeDimOp(self,node,slice):
        return slice.modify(rtype=node.type,dims=node.dims)

    def visitDetectFixedShapesOp(self,node,slice):
        data = slice.data.flat()
        if len(data) == 0:
            raise RuntimeError, "Cannot determine dim shape from empty data. Please cast dim shape of fixed dims."
        ntype = detect_shape(data[0],slice.type)
        return slice.modify(rtype=ntype)
    
    def visitDetectTypeOp(self,node,slice):
        if(slice.type == rtypes.unknown):
            det = detector.Detector()
            det.setParentDimensions(node.dims)
            det.processSeq(slice.data.flat())
            return slice.modify(rtype=det.getType())
        else:
            return slice
    
    def visitDetectAndCastOp(self,node,slice):
        det = detector.Detector(allow_need_convert=node.allow_convert)
        det.setParentDimensions(node.dims)
        det.processSeq(slice.data.flat())
        nslice = slice.modify(rtype=det.getType())
        return self.unpackCast(nslice.type, slice, nslice)

    def unpackCastTypeTuple(self, rtype, oslice, nslice):
        nslices = []
        for i in range(len(rtype.fieldnames)):
            tnnode = ops.UnpackTupleOp(nslice, i)
            toslice = self.visitUnpackTupleOp(tnnode, oslice)
            toslice = self.unpackCast(tnnode.type, toslice, tnnode)
            nslices.append(toslice)
        newnode = ops.PackTupleOp(tuple(nslices), nslice.name)
        return self.visitPackTupleOp(newnode, nslices)

    def unpackCastTypeArray(self, rtype, oslice, nslice):
        nnode = ops.UnpackArrayOp(nslice)

        toslice = self.visitUnpackArrayOp(nnode, oslice)
        toslice = self.unpackCast(nnode.type, toslice, nnode)
        
        onode = ops.PackArrayOp(toslice)
        return self.visitPackArrayOp(onode, toslice)
    
    def unpackCastTypeSet(self, rtype, oslice, nslice):
        nnode = ops.UnpackArrayOp(nslice)

        toslice = self.visitUnpackArrayOp(nnode, oslice)
        toslice = self.unpackCast(nnode.type, toslice, nnode)
        
        onode = ops.UnaryFuncAggregateOp('Set',repops_funcs.setsig, repops_funcs.Param(nslice.name, nslice.type), 1, toslice)
        return self.visitUnaryFuncAggregateOp(onode, toslice)
    
    
    def unpackCastTypeScalar(self, rtype, oslice, nslice):
        newnode = ops.CastOp(oslice, nslice.type)
        return self.visitCastOp(newnode, oslice)
    
    unpackCastTypeString = unpackCastTypeScalar
    unpackCastTypeAny = unpackCastTypeScalar

    def visitUnpackArrayOp(self,node,slice):
        ndata=slice.data.unpack(node.unpack_dims, subtype=node.type)
        return slice.modify(data=ndata,rtype=node.type,dims=node.dims)

    def visitPackArrayOp(self, node, slice):
        ndata=slice.data.pack(node.type, len(node.pack_dims))
        return slice.modify(data=ndata,rtype=node.type,dims=node.dims)

    def visitInsertDimOp(self,node,slice):
        ndata = slice.data.insertDim(node.matchpoint)
        return slice.modify(data=ndata,rtype=node.type,dims=node.dims)

    def visitPermuteDimsOp(self, node, slice):
        ndata=slice.data.permuteDims(node.permute_idxs)
        return slice.modify(data=ndata,dims=node.dims,rtype=node.type)

    def visitSplitDimOp(self, node, slice):
        ndata=slice.data.splitDim(node.pos,node.lshape,node.rshape)
        return slice.modify(data=ndata,dims=node.dims,rtype=node.type)

    def visitPackListOp(self, node, slice):
        ndata=slice.data.pack(node.type, len(node.pack_dims))
        func = lambda x: x.tolist()
        ndata=ndata.map(func,res_type=node.type)
        return slice.modify(data=ndata,rtype=node.type,dims=node.dims)

    def visitToPythonOp(self,  node, slice):
        func = lambda x: convertors.rpc_convertor.execConvert(slice.type,x)
        ndata = slice.data.mapseq(func,res_type=node.type)
        return slice.modify(data=ndata, rtype=node.type)

    def visitUnpackTupleOp(self,node,slice):
        if(isinstance(slice.type,rtypes.TypeRecordDict)):
            didx = slice.type.fieldnames[node.tuple_idx]
            if(node.type.has_missing):
                func = lambda x: x.get(didx,Missing)
            else:
                func = operator.itemgetter(didx)
        elif(isinstance(slice.type,rtypes.TypeIndexDict)):
            if node.tuple_idx == 0:
                func = lambda x: x.keys()
            else:
                func = lambda x: x.values()
        else:
            if(node.type.has_missing):
                def func(x):
                    if node.tuple_idx >= len(x):
                        return Missing
                    else:
                        return x[node.tuple_idx]
            else:
                func = operator.itemgetter(node.tuple_idx)
        ndata = slice.data.map(func,res_type=node.type)
        return slice.modify(data=ndata,rtype=node.type,name=node.name)
 
    def visitPackTupleOp(self,node, slices):
        ndata = nested_array.co_mapseq(speedtuplify,[slice.data for slice in slices],res_type=node.type)
        return slices[0].modify(data=ndata,name=node.name,rtype=node.type,dims=node.dims,bookmarks=node.bookmarks)
   
    def visitPackIndexDictOp(self, node, slices):
        names = node.type.fieldnames
        def speed_index_dictify(x):
            return dict(zip(*x))
        ndata = nested_array.co_map(speed_index_dictify,[slice.data for slice in slices],res_type=node.type)
        return slices[0].modify(data=ndata,name=node.name,rtype=node.type,dims=node.dims,bookmarks=node.bookmarks)
       
    def visitTakeOp(self, node, slices):
        ndata = nested_array.co_mapseq(speeddictindex,[slice.data for slice in slices],res_type=node.type, 
                                        dtype=node.type.toNumpy(), allow_missing=node.allow_missing, keep_missing = node.keep_missing)
        return slices[0].modify(data=ndata,name=node.name,rtype=node.type,dims=node.dims,bookmarks=node.bookmarks)

    def visitPackDictOp(self,node, slices):
        names = node.type.fieldnames
        if node.with_missing or not any([slice.type.has_missing for slice in slices]):
            def speeddictify(x):
                return util.darray([dict(zip(names,row)) for row in zip(*x)])
        else:
            def speeddictify(x):
                non_hasmissing = []
                for pos, slice in enumerate(slices):
                    if not slice.type.has_missing:
                        non_hasmissing.append(pos)
                if(non_hasmissing):
                    nx = util.darray(x)[non_hasmissing] 
                    snames = util.darray(names)[non_hasmissing]
                    d = [dict(zip(snames,row)) for row in zip(*nx)]
                else:
                    d = [{} for i in xrange(len(x[0]))]
                for pos, slice in enumerate(slices):
                    if(pos in non_hasmissing):
                        continue
                    name = names[pos]
                    val = x[pos]
                    for rowpos in xrange(len(val)):
                        elem = val[rowpos]
                        if not elem is Missing:
                            d[rowpos][name] =elem
                return util.darray(d)

        ndata = nested_array.co_mapseq(speeddictify,[slice.data for slice in slices],res_type=node.type)
        return slices[0].modify(data=ndata,name=node.name,rtype=node.type,dims=node.dims,bookmarks=node.bookmarks)
   
    def visitHArrayOp(self,node, slices):
        ndata = nested_array.co_mapseq(speedarrayify,[slice.data for slice in slices],dtype=node.type.subtypes[0].toNumpy(), res_type=node.type)
        return slices[0].modify(data=ndata,name=node.name,rtype=node.type,dims=node.dims,bookmarks=node.bookmarks)

    def visitEnsureCommonDimOp(self,node,slice,compare_slice):
        checkdim = node.dims[node.checkpos]
        selfshape = slice.data.getDimShape(node.checkpos)

        otherpos = compare_slice.dims.index(checkdim)
        othershape = compare_slice.data.getDimShape(otherpos)
        assert numpy.all(selfshape == othershape), "Dimension mismatch in " + str(checkdim) + ":" + str(selfshape) + " != " + str(othershape)

        return slice.modify(rtype=node.type,dims=node.dims)

    def visitShapeOp(self, node, slice):
        d = slice.data.getDimShape(node.pos)
        ndata = nested_array.NestedArray(d,node.type)
        return slice.modify(ndata,rtype=node.type,dims=node.dims,name=node.name)

    def visitFreezeOp(self, node, slice):
        func = lambda x: type_attribute_freeze.freeze_protocol.execFreeze(slice.type,x)
        ndata = slice.data.mapseq(func,res_type=node.type)
        return slice.modify(data=ndata)

    def visitBroadcastOp(self,node,slice,compare_slices):
        repeat_dict = {}
        bcpos = 0
        for pos,planelem in enumerate(node.plan):
            if(planelem == BCEXIST):
                dimpos = compare_slices[bcpos].dims.index(node.bcdims[pos])
                
                rshape = compare_slices[bcpos].data.getDimShape(dimpos)
                #if it is a variable shape, keep only shape dims that are variable
                if(not isinstance(rshape,int)):
                    dim = compare_slices[bcpos].dims[dimpos]
                    rshape = remove_independent(rshape,dim)
                    rshape = add_independent(rshape,node.dims[pos])
                repeat_dict[pos] = rshape
                bcpos += 1
            elif(planelem == BCCOPY or planelem == BCSOURCE):
                pass
            else:
                raise RuntimeError, "Unknown broadcast plan element: " + str(planelem)
        ndata = slice.data.broadcast(repeat_dict)
        return slice.modify(data=ndata,dims=node.dims)


    def visitFilterOp(self,node, slice, constraint):
        ndata = nested_array.co_map(speedfilter,[slice.data, constraint.data],
                                       has_missing = node.has_missing,ctype=constraint.type,stype=node.type,
                                       res_type=node.type)
        return slice.modify(data=ndata,rtype=node.type, dims=node.dims)

    def visitFlatAllOp(self, node, slice):
        ndata = slice.data.mergeAllDims()
        return slice.modify(data=ndata,rtype=node.type,dims=node.dims)
    
    def visitFlatDimOp(self, node, slice):
        ndata = slice.data.mergeDim(node.flatpos-1,result_fixed=len(node.dims[node.flatpos-1].dependent) == 0)
        return slice.modify(data=ndata,rtype=node.type,dims=node.dims)


    def visitGroupIndexOp(self, node, slices):
        ndata = nested_array.co_map(groupindex,[slice.data for slice in slices],
                                        res_type = node.type)
        
        return slices[0].modify(data=ndata,name=node.name,rtype=node.type,dims=node.dims,bookmarks=node.bookmarks)

    def visitUnaryFuncElemOp(self,node, slice):
        try:
            func = getattr(self, node.sig.name + node.funcname)
        except AttributeError:
            func = getattr(self, node.sig.name + "General")
        

        ndata = slice.data.mapseq(func,type_in=slice.type,type_out=node.type,
                                  res_type=node.type,op=node.funcname, **node.kwargs)
        return slice.modify(data=ndata,rtype=node.type)
    
    def visitNoneToMissingOp(self, node, slice):
        ndata = slice.data.mapseq(none_to_missing,res_type=node.type, stype=node.type)
        return slice.modify(data=ndata)

    def visitUnaryFuncSeqOp(self,node, slice):
        try:
            func = getattr(self, node.sig.name + node.funcname)
        except AttributeError:
            func = getattr(self, node.sig.name + "General")
       
        ndata = slice.data
        if(node.packdepth > 2):
            ndata,nshapes = ndata.mergeLastDims(node.packdepth-2)
            dim = dimensions.Dim(UNDEFINED,(True,))
            ldims = dimpaths.DimPath(slice.dims[-node.packdepth],dim)
        else:
            ldims = slice.dims[-node.packdepth:]
        
        ndata = ndata.pack(slice.type, min(node.packdepth,2))
        ndata = ndata.mapseq(func,type_in=slice.type,type_out=node.type,
                                  res_type=node.type,op=node.funcname, packdepth=node.packdepth, **node.kwargs)
        ndata = ndata.unpack(ldims, subtype=node.type)
        if(node.packdepth > 2):
            ndata = ndata.splitLastDim(nshapes)

        return slice.modify(data=ndata,rtype=node.type)

       
    def visitUnaryFuncAggregateOp(self,node, slice):
        try:
            func = getattr(self, node.sig.name + node.funcname)
        except AttributeError:
            func = getattr(self, node.sig.name + "General")
        ndata = slice.data
        if(node.packdepth > 1):
            if(node.packdepth > 2):
                ndata,nshapes = ndata.mergeLastDims(node.packdepth-2)
            ndata = ndata.pack(slice.type, 2)
        elif(node.packdepth == 1):
            ndata = ndata.pack(slice.type)
        else: #packdepth == 0
            ndata = ndata
        ndata = ndata.mapseq(func,type_in=slice.type,type_out=node.type,
                                  res_type=node.type,op=node.funcname,packdepth=node.packdepth, **node.kwargs)
        
        if(node.packdepth > 1):
            dim = dimensions.Dim(UNDEFINED,(True,))
            ndata = ndata.unpack(dimpaths.DimPath(dim), subtype=node.type)
            if(node.packdepth > 2):
                nshapes = nested_array.drop_prev_shapes_dim(ndata,nshapes)
                ndata = ndata.splitLastDim(nshapes)
            
        return slice.modify(data=ndata,rtype=node.type,dims=node.dims)



    def visitBinFuncElemOp(self,node, slices):
        try:
            func = getattr(self, node.sig.name + node.funcname)
        except AttributeError:
            func = getattr(self, node.sig.name + "General")

        ndata = nested_array.co_mapseq(func,[slice.data for slice in slices],
                                       type1=slices[0].type,type2=slices[1].type,
                                       typeo=node.type,res_type=node.type,op=node.funcname)

        return slices[0].modify(data=ndata,name=node.name,rtype=node.type,dims=node.dims,bookmarks=node.bookmarks)

    def visitEquiJoinIndexOp(self, node, slices):
        ndatas = nested_array.co_map(joinindex,[slice.data for slice in slices],
                                       jointype=node.jointype,
                                       res_type=(node.results[0].type,node.results[1].type))
        return slices[0].modify(data=ndatas,name=None,rtype=None,dims=None, bookmarks=None)

    def visitSelectOp(self, node, slice):
        return slice.modify(data=slice.data[node.index],name=node.name, rtype=node.type, dims=node.dims, bookmarks=node.bookmarks)

    def visitFixateOp(self,node,slices=[]):
        res = []
        for cur_slice, slice in zip(node.sources, slices):
            ndata = slice.data.getStructuredData()
            nslice = slice.modify(data=ndata)
            nslice.setSource(cur_slice)
            res.append(nslice)
        return res
    
    def visitGatherOp(self,node,slices):
        res = []
        for cur_slice, slice in zip(node.sources, slices):
            slice.setSource(cur_slice)
            res.append(slice)
        return res
   
    def castto_any(self,castname,node,slice):
        dtype = node.type.toNumpy()
        return slice.data.mapseq(lambda x:numpy.cast[dtype](x),res_type=node.type)
    
    def castto_pickle(self,castname,node,slice):
        dtype = node.type.toNumpy()
        return slice.data.map(lambda x:cPickle.dumps(x,2),res_type=node.type)
    
    def castfrom_pickle(self,castname,node,slice):
        dtype = node.type.toNumpy()
        return slice.data.map(lambda x:cPickle.loads(str(x)),res_type=node.type)

    def castto_numbers(self,castname,node,slice):
        dtype = node.type.toNumpy()
        if not node.type.has_missing:
            return slice.data.mapseq(lambda x:numpy.cast[dtype](x),res_type=node.type)
        else:
            xtype = dtype.type
            def val(v):
                try:
                    return xtype(v)
                except ValueError:
                    return Missing

            return slice.data.map(val, res_type=node.type)
            #raise RuntimeError, "Casting missing values not yet supported"

    def caststring_to_int(self, castname, node, slice):
        if(node.type.has_missing):
            func = string_to_int_missing
        else:
            func = string_to_int
        dtype = node.type.toNumpy()

        return slice.data.mapseq(lambda x: func(x,dtype),res_type=node.type)
    
    def caststring_to_string(self, castname, node, slice):
        return slice.data

    def castto_array(self, castname, node, slice):
        func = stringset_to_array
        dtype = node.type.toNumpy()
        return slice.data.mapseq(func, dtype=dtype, res_type=node.type)
    
    
    def castto_bytes(self, castname, node, slice):
        if(node.type.has_missing):
            func = any_tobytes_missing
        else:
            func = any_tobytes
        return slice.data.mapseq(func, res_type=node.type)

    def caststring_to_real(self, castname, node, slice):
        if(node.type.has_missing):
            func = string_to_real_missing
        else:
            func = string_to_real
        dtype = node.type.toNumpy()

        return slice.data.mapseq(lambda x: func(x,dtype),res_type=node.type)
       
    def caststring_to_string(self, castname, node, slice):
        if(node.type.has_missing):
            func = string_to_string_missing
        else:
            func = string_to_string
        dtype = node.type.getPythonType()

        return slice.data.mapseq(lambda x: func(x,dtype),res_type=node.type)

    def withinWithin(self, data, type1, type2, typeo, op, bcdepth=1):
        data1,data2 = data
        if(bcdepth > 1):
            data2 = util.darray([set(elem) for elem in data2.ravel()]).reshape(data2.shape)
            zipdata = numpy.broadcast(data1,data2)
        else:
            zipdata = zip(data1,data2)
        res = [elem in arr for elem,arr in zipdata]
        return util.darray(res,bool)

    def withinContains(self, data, type1, type2, typeo, op, bcdepth=1):
        data2,data1 = data
        if(bcdepth > 1):
            data2 = util.darray([set(elem) for elem in data2.ravel()]).reshape(data2.shape)
            zipdata = numpy.broadcast(data1,data2)
        else:
            zipdata = zip(data1,data2)
        res = [elem in arr for elem,arr in zipdata]
        return util.darray(res,bool)
    
    def mergeMerge(self, data, type1, type2, typeo, op):
        data1, data2 = data
        res = []
        for lelem, relem in zip(data1, data2):
            if lelem is relem:
                res.append(lelem)
            elif lelem is Missing:
                res.append(relem)
            elif relem is Missing:
                res.append(lelem)
            elif lelem == relem:
                res.append(lelem)
            else:
                raise RuntimeError, "Found unequal values during merge: " + str(lelem) + " != " + str(relem)
        return util.darray(res, typeo.toNumpy())

    def simple_arithDivide(self, data, type1, type2, typeo, op):
        data1,data2 = data
        if(data1 is Missing or data2 is Missing):
            return Missing
        filter = (data2 == 0) & (data1 == 0)
        if filter.any():
            data2 = data2.copy()
            data2[filter] = 1.0
        if util.numpy16up:
            res = numpy_arith[op](data1, data2, dtype=typeo.toNumpy())
        else:
            res = numpy_arith[op](data1, data2, sig=typeo.toNumpy())
        return res

    def simple_arithGeneral(self, data, type1, type2, typeo, op):
        data1,data2 = data
        if(data1 is Missing or data2 is Missing):
            return Missing
        if util.numpy16up:
            return numpy_arith[op](data1, data2, dtype=typeo.toNumpy())
        else:
            return numpy_arith[op](data1, data2, sig=typeo.toNumpy())
    boolboolGeneral = simple_arithGeneral

    def string_add_stringAdd(self, data, type1, type2, typeo, op):
        data1,data2 = data
        if(data1 is Missing or data2 is Missing):
            return Missing
        return util.darray(list(numpy_arith[op](numpy.cast[object](data1), numpy.cast[object](data2))),typeo.toNumpy())
    
    def array_add_arrayAdd(self, data, type1, type2, typeo, op):
        data1,data2 = data
        if(data1 is Missing or data2 is Missing):
            return Missing
        res = []
        dtype = typeo.toNumpy()
        for lelem, relem in zip(data1,data2):
            mshape = max(1,min(len(getattr(lelem,'shape',(0,))), len(getattr(relem,'shape',(0,)))))
            res.append(numpy.concatenate([util.darray(list(lelem),dtype,mshape),util.darray(list(relem),dtype,mshape)],axis=0))
        return util.darray(res,object)

    def simple_cmpGeneral(self, data, type1, type2, typeo, op):
        #a numpy bug gives all true arrays when using
        #bool as outtype in comparison
        return numpy_cmp[op](data[0], data[1])

    def string_cmpGeneral(self, data, type1, type2, typeo, op):
        #a numpy bug gives NotImplemented when performing operations,
        #such as "numpy.equal" on string arrays
        #so use direct operations ("__eq__")
        op = python_op[op]
        #Note: numpy seems to segfault, when using reverse operations on string comparision
        #using as first argument an  object array and as second a string array. 
        if(isinstance(data[0],numpy.ndarray) and data[0].dtype == object and isinstance(data[1],numpy.ndarray) and data[1].dtype != object):
            data = (data[0], numpy.cast[object](data[1]))

        res = getattr(data[0], op)(data[1])
        if(res is NotImplemented):
            if(isinstance(data[0],numpy.ndarray) and data[0].dtype != object and isinstance(data[1],numpy.ndarray) and data[1].dtype == object):
                data = (numpy.cast[object](data[0]), data[1])
            res = getattr(data[1], reverse_op[op])(data[0])
        assert not res is NotImplemented, "Not implemented error in stringstringGeneral for " \
                                            + str(op) + " and " + str(type1) + ", " + str(type2)
        return res

    def numberGeneral(self, data, type_in, type_out, op):
        if util.numpy16up:
            return numpy_unary_arith[op](data, dtype=type_out.toNumpy())
        else:
            return numpy_unary_arith[op](data, sig=type_out.toNumpy())
    
    def boolInvert(self, data, type_in, type_out, op):
        res = []
        for elem in data:
            if elem is Missing:
                res.append(elem)
            else:
                res.append(not elem)
        return util.darray(res, type_out.toNumpy())

    def repmissingReplaceMissing(self, data, type_in, type_out, op, def_value=NOVAL):
        if not type_in.has_missing:
            return data
        if def_value is NOVAL:
            if type_out.__class__ is rtypes.TypeArray and type_out.dims[0].shape == UNDEFINED:
                for elem in data:
                    if not elem is Missing:
                        shape = len(elem)
                        break
                else:
                    raise RuntimeError, "Could not determine shape of dimension. ReplaceMissing impossible. Change to variable dim."
               
                dim = dimensions.Dim(shape, type_out.dims[0].dependent)
                def_value = dimpaths.dimsToArrays(dimpaths.DimPath(dim),type_out.subtypes[0]).toDefval()
            else:                
                def_value = type_out.toDefval()
        res = []
        for elem in data:
            if elem is Missing:
                res.append(def_value)
            else:
                res.append(elem)
        return util.darray(res, type_out.toNumpy())
    
    def ismissingIsMissing(self, data, type_in, type_out, op, def_value=NOVAL):
        res = []
        for elem in data:
            if elem is Missing:
                res.append(True)
            else:
                res.append(False)
        return util.darray(res, type_out.toNumpy())
   
    def corrCorr(self, data, type_in, type_out, op):
        intype = type_in.toNumpy()
        res = []
        for elem in data:
            if(len(elem.shape) < 2):
                elem = util.darray(list(elem),intype,2,2)
            res.append(numpy.corrcoef(elem))
        return util.darray(res,object,1,1)

    def eachEach(self, data, type_in, type_out, op, eachfunc):
        if(isinstance(eachfunc,context.Context)):
            return util.darray([context._apply(eachfunc,elem) for elem in data],type_out.toNumpy())
        else:
            return util.darray([eachfunc(elem) for elem in data],type_out.toNumpy())

    def sortableArgsort(self, data, type_in, type_out, op, packdepth, descend=False):
        data = ensure_fixeddims(data,packdepth,type_in.toNumpy())
        if(len(data.shape) < 2):
            if(descend):
                res = util.darray([numpy.flipud(numpy.argsort(row,axis=0)) for row in data],object)
            else:
                res = util.darray([numpy.argsort(row,axis=0) for row in data],object)
        else:
            res = numpy.argsort(data,axis=1)
            if(descend):
                res = res[:,::-1,...]
        return res
    
    def sortableRank(self, data, type_in, type_out, op, packdepth, descend=False):
        data = ensure_fixeddims(data,packdepth,type_in.toNumpy())
        if(len(data.shape) < 2):
            if(descend):
                res = util.darray([numpy.argsort(numpy.flipud(numpy.argsort(row,axis=0)),axis=0) for row in data],object)
            else:
                res = util.darray([numpy.argsort(numpy.argsort(row,axis=0),axis=0) for row in data],object)
        else:
            res = numpy.argsort(data,axis=1)
            if(descend):
                res = res[:,::-1,...]
            res = numpy.argsort(res,axis=1)                
        return res
    
    def fixdimCumSum(self, data, type_in, type_out, op, packdepth):
        data = ensure_fixeddims(data,packdepth,type_in.toNumpy())
        if(len(data.shape) < 2):
            res = util.darray([numpy.cumsum(row,axis=0) for row in data],object)
        else:
            res = numpy.cumsum(data,axis=1)
        return res
 
    def any_nodepPos(self, data, type_in, type_out, op, packdepth):
        dtype = type_out.toNumpy()
        if(len(data.shape) == 1):
            if(packdepth == 1):
                xres = [numpy.arange(len(row),dtype=dtype) for row in data]
            else:
                xres = []
                for row in data:
                    res = numpy.arange(len(row),dtype=dtype)
                    if(len(row.shape) > 1):
                        res = numpy.repeat(res,row.shape[1]).reshape(row.shape)
                    else:
                        subres = []
                        for r,d in zip(res.ravel(),row.ravel()):
                            subres.append(util.darray([r] * len(d),dtype))
                        res = util.darray(subres,object)
                    xres.append(res)
            xres = util.darray(xres)
        elif(len(data.shape) == 2):
            xres = numpy.tile(numpy.arange(data.shape[1],dtype=dtype),data.shape[0]).reshape(data.shape[:2])
            if(packdepth > 1):
                res = []
                for r,d in zip(xres.ravel(),data.ravel()):
                    res.append(util.darray([r] * len(d),dtype))
                xres = util.darray(res,object,1,1).reshape(data.shape)
        else:
            assert len(data.shape) == 3, "Unexpected data shape"
            r =  numpy.tile(numpy.arange(data.shape[1],dtype=dtype),data.shape[0]).reshape(data.shape[:2])
            xres = numpy.repeat(r,data.shape[2]).reshape(data.shape)
           
        return xres

    def countCount(self, data, type_in, type_out, op, packdepth):
        dtype = type_out.toNumpy()
        if(len(data.shape) == 1):
            return util.darray([len(row) for row in data],dtype)
        else:
            return util.darray([data.shape[1]] * data.shape[0],dtype)
    
    def setGeneral(self, data, type_in, type_out, op, packdepth):
        data = ensure_fixeddims(data,packdepth,type_in.toNumpy())
        dtype = type_out.toNumpy()
        if(packdepth > 1):
           return util.darray([[set(subrow) for subrow in row.transpose()] for row in data],dtype,2,2)
        else:
           return util.darray([set(row) for row in data],dtype)
    
    def uniqueGeneral(self, data, type_in, type_out, op, packdepth):
        data = ensure_fixeddims(data,packdepth,type_in.toNumpy())
        dtype = type_out.toNumpy()
        if(packdepth > 1):
           return util.darray([[numpy.unique(subrow,return_index=True)[1] for subrow in row.transpose()] for row in data],dtype,2,2)
        else:
           return util.darray([numpy.unique(row, return_index=True)[1] for row in data],dtype)
   
    def arrayarraySum(self, data, type_in, type_out, op, packdepth):
        data = ensure_fixeddims(data,packdepth,type_in.toNumpy())
        if(packdepth > 1):
            return util.darray([[numpy.concatenate(list(subrow),axis=0) for subrow in row.transpose()] for row in data],object,2,2)
        else:
            return util.darray([numpy.concatenate(list(row),axis=0) for row in data],object)
    
    def stringstringSum(self, data, type_in, type_out, op, packdepth):
        data = ensure_fixeddims(data,packdepth,type_in.toNumpy())
        if(packdepth > 1):
            return util.darray([[''.join(list(subrow)) for subrow in row.transpose()] for row in data],object,2,2)
        else:
            return util.darray([''.join(list(row)) for row in data],object)
    
    def fixdimArgmax(self, data, type_in, type_out, op, packdepth):
        data = ensure_fixeddims(data,packdepth,type_in.toNumpy())
        func = numpy_dimfuncs[op]
        dtype = type_out.toNumpy()
        if type_in.has_missing:               
            res = []
            for row in data:
                pos = numpy.arange(len(row))
                filter = ~numpy.equal(row, Missing)
                pos = pos[filter]
                row = row[filter]
                res.append(pos[func(row,axis=0)])
            return util.darray(res,dtype)
        else:            
            if(len(data.shape) < 2):
                if(packdepth > 1):
                    dtype = object
                return util.darray([func(row,axis=0) for row in data],dtype)
            else:
                return func(data,axis=1)
    fixdimArgmin = fixdimArgmax

    def fixdimGeneral(self, data, type_in, type_out, op, packdepth):
        data = ensure_fixeddims(data,packdepth,type_in.toNumpy())
        func = numpy_dimfuncs[op]
        dtype = type_out.toNumpy()
        if type_in.has_missing:                
            return util.darray([func([elem for elem in row if not elem is Missing],axis=0) for row in data],dtype)
        else:            
            if(len(data.shape) < 2):
                if(packdepth > 1):
                    dtype = object
                return util.darray([func(row,axis=0) for row in data],dtype)
            else:
                return func(data,axis=1)
      
numpy_cmp = {'Equal':numpy.equal,#{{{
            'NotEqual':numpy.not_equal,
            'LessEqual':numpy.less_equal,
            'GreaterEqual':numpy.greater_equal,
            'Less':numpy.less,
            'Greater':numpy.greater}

numpy_arith = { 'Add':numpy.add,
                'Subtract':numpy.subtract,
                'Multiply':numpy.multiply,
                'Modulo':numpy.mod,
                'Divide':numpy.divide,
                'FloorDivide':numpy.floor_divide,
                'And':numpy.bitwise_and,
                'Or':numpy.bitwise_or,
                'Xor':numpy.bitwise_xor,
                'Power':numpy.power
                }
numpy_dimfuncs = {
    'Max':numpy.max,
    'Min':numpy.min,
    'Mean':numpy.mean,
    'Any':numpy.any,
    'All':numpy.all,
    'Sum':numpy.sum,
    'Median':numpy.median,
    'Argmin':numpy.argmin,
    'Argmax':numpy.argmax,
    'Std':numpy.std
    }

python_op = {'Equal':'__eq__',
             'NotEqual':'__ne__',
             'LessEqual':'__le__',
             'GreaterEqual':'__ge__',
             'Less':'__lt__',
             'Greater':'__gt__',
             'Add':'__add__',
             'Subtract':'__sub__',
             'Multiply':'__mul__',
             'Modulo':'__mod__',
             'Divide':'__div__',
             'FloorDivide':'__floordiv__',
             'And':'__and__',
             'Or':'__or__',
             'Xor':'__xor__',
             'Power':'__pow__'}

numpy_unary_arith = {
    "Invert":numpy.invert,
    "Negative":numpy.negative,
    "Abs":numpy.abs
    }

reverse_op = {'__eq__':'__eq__',
            '__ne__':'__ne__',
            '__le__':'__ge__',
            '__ge__':'__le__',
            '__lt__':'__gt__',
            '__gt__':'__lt__',
            '__add__':'__radd__',
            '__radd__':'__add__',
            '__sub__':'__rsub__',
            '__rsub__':'__sub__',
            '__mul__':'__rmul__',
            '__rmul__':'__mul__',
            '__mod__':'__rmod__',
            '__rmod__':'__mod__',
            '__div__':'__rdiv__',
            '__rdiv__':'__div__',
            '__floordiv__':'__rfloordiv__',
            '__rfloordiv__':'__floordiv__',
            '__and__':'__rand__',
            '__rand__':'__and__',
            '__or__':'__ror__',
            '__ror__':'__or__',
            '__xor__':'__rxor__',
            '__rxor__':'__xor__',
            '__pow__':'__rpow__',
            '__rpow__':'__pow__'
            }#}}}

#util funcs
def speedtuplify(seqs):
    nseq = util.darray(zip(*seqs))
    return nseq


def speedarrayify(seqs,dtype):
    nseq = numpy.array(seqs,dtype).T
    return nseq


def speeddictindex(seqs,dtype, allow_missing=False, keep_missing=False):
    if keep_missing:
        return util.darray([d.get(k,k) for d, k in zip(*seqs)],dtype)
    elif allow_missing:
        return util.darray([d.get(k,Missing) for d, k in zip(*seqs)],dtype)
    else:
        return util.darray([d[k] for d, k in zip(*seqs)],dtype)


def speedfilter(seqs,has_missing, ctype, stype):
    data,constraint = seqs
    if data is Missing:
        return data
    if len(data) == 0: #FIXME
        return data
    if(has_missing):
        if(isinstance(ctype,rtypes.TypeArray)):
            missing = stype.subtypes[0].toMissingval()
            if(isinstance(ctype.subtypes[0],rtypes.TypeBool)):
                res = []
                for pos, elem in enumerate(constraint.ravel()):
                    if(elem is Missing):
                        res.append(missing)
                    elif(elem == True):
                        res.append(data[pos])
                res = util.darray(res,object)
            else:#indices
                res = []
                for elem in constraint.ravel():
                    if(elem is Missing):
                        res.append(missing)
                    else:
                        res.append(data[elem])
                res = util.darray(res,object)
        else:
            if(constraint is Missing):
                missing = stype.toMissingval()
                res = missing
            else:
                try:
                    res = data[constraint]
                except Exception:
                    res = util.darray(data)[constraint]
    else:
        try:
            res = data[constraint]
        except Exception:
            res = util.darray(data)[constraint]
    return res

def ensure_fixeddims(seqs,packdepth,dtype):
    if(packdepth > 1):
        if(len(seqs.shape) == 1):
            res = []
            for seq in seqs:
                if(len(seq.shape) >= 2):
                    res.append(seq)
                else:
                    res.append(util.darray(seq,dtype,100000,2))
            res = util.darray(res,object)
        elif(len(seqs.shape) == 2):
            res = util.darray(seqs.tolist(),dtype,1000,3)
        else:
            res = seqs
    else:
        res = seqs
    return res


def groupindex(data):
    indexes = []
    if(len(data) > 1):
        for dcol in data:
            index = {}
            pos = 0 
            for elem in dcol:
                if not elem in index:
                    index[elem] = pos
                    pos += 1
            indexes.append(index)
        
        data_dict = defaultdict(list)
        for pos, elems in enumerate(zip(*data)):
            data_dict[elems].append(pos)
       
        shape = [len(index) for index in indexes]
        indexdata = [util.darray([],int)] * numpy.prod(shape)
        indexdata = util.darray(indexdata, object, 1).reshape(tuple(shape))
        
        for key, posses in data_dict.iteritems():
            loc = tuple([index[keypart] for index, keypart in zip(indexes,key)])
            indexdata[loc] = util.darray(posses, int)
    else:
        data_dict = defaultdict(list)
        for pos, elems in enumerate(*data):
            if(elems is Missing):
                data_dict["__TEMP__" + str(elems.__hash__())].append(pos)
            else:
                data_dict[elems].append(pos)
        indexdata = [util.darray(elem, int) for elem in data_dict.values()]
        indexdata = util.darray(indexdata, object, 1)
   
    return indexdata


def create_elem_pos_dict(data):
    res = {}
    for pos, elem in enumerate(data):
        if elem in res:
            res[elem].append(pos)
        else:
            res[elem] = [pos]
    return res

def joinindex(data, jointype):
    lelempos = create_elem_pos_dict(data[0])
    relempos = create_elem_pos_dict(data[1])

    tlpos = []
    trpos = []
    leftouter = (jointype == "left" or jointype == "full")
    rightouter = (jointype == "right" or jointype == "full")
    for elem,lpos in lelempos.iteritems():
        if not elem in relempos:
            if(leftouter):
                tlpos.extend(lpos)
                trpos.extend([Missing] * len(lpos))
            continue
        rpos = relempos[elem]
        if(len(lpos) == 1):
            if len(rpos) == 1:
                tlpos.append(lpos[0])
                trpos.append(rpos[0])
            else:
                tlpos.extend(lpos * len(rpos))
                trpos.extend(rpos)
        else:
            if len(rpos) == 1:
                tlpos.extend(lpos)
                trpos.extend(rpos * len(lpos))
            else:
                tlpos.extend(lpos * len(rpos))
                trpos.extend(numpy.repeat(rpos,len(lpos)))
    
    if(rightouter):
        unused_keys = list(set(range(len(data[1]))) - set(trpos))
        tlpos.extend([Missing] * len(unused_keys))
        trpos.extend(unused_keys)
        ldtype = object
    else:
        ldtype = int

    if(leftouter):
        rdtype = object
    else:
        rdtype = int

    return (util.darray(tlpos,ldtype), util.darray(trpos,rdtype))

def remove_independent(data,dim):
    wx = [0] * len(data.shape)
    for pos, dep in enumerate(dim.dependent):
        #FIXME: handle variable prev dims (move to nested_array)
        if dep and len(wx) > pos:
            wx[-(pos+1)] = slice(None,None)
    data = data[wx]
    return data

def add_independent(data,dim):
    wx = list(data.shape)[::-1]
    for pos, dep in enumerate(dim.dependent):
        if not dep:
            wx.insert(pos,1)
    data = numpy.reshape(data,wx[::-1])
    return data


def stringset_to_array(seq, dtype):
    res = []
    for elem in seq:
        if elem is Missing:
            res.append(Missing)
        else:
            res.append(util.darray(list(elem),dtype))
    return util.darray(res)


def any_tobytes_missing(seq):
    res = []
    for elem in seq:
        if elem is Missing:
            res.append(Missing)
        else:
            res.append(str(elem))
    return util.darray(res)

def any_tobytes(seq):
    res = [str(elem) for elem in seq.astype(object)]
    return util.darray(res)


def string_to_int_missing(seq, dtype):
    res = []
    for elem in seq:
        if elem is Missing or elem == "":
            res.append(Missing)
        else:
            try:
                res.append(int(elem))
            except ValueError:
                res.append(Missing)
    return util.darray(res,dtype)

def string_to_int(seq, dtype):
    return util.darray([int(elem) for elem in seq],dtype)


def string_to_real_missing(seq, dtype):
    res = []
    for elem in seq:
        if elem is Missing or elem == "":
            res.append(Missing)
        else:
            try:
                res.append(float(elem))
            except ValueError:
                res.append(Missing)
    return util.darray(res,dtype)

def string_to_real(seq, dtype):
    return util.darray([float(elem) for elem in seq],dtype)

def string_to_string_missing(seq, dtype):
    res = []
    for elem in seq:
        if elem is Missing or elem == "":
            res.append(Missing)
        else:
            try:
                res.append(dtype(elem))
            except ValueError:
                res.append(Missing)
    return util.darray(res,object)

def string_to_string(seq, dtype):
    return util.darray([dtype(elem) for elem in seq],object)


def none_to_missing(seq,stype):
    missing = stype.toMissingval()
    seq = seq.copy()
    seq[numpy.equal(seq,None)] = missing
    return seq
   
def detect_shape(data, stype):
    if stype.__class__ is rtypes.TypeArray:
        if not stype.dims[0].dependent and stype.dims[0].shape == UNDEFINED:
            stype.dims[0].shape = len(data)

        subtype = detect_shape(data[0],stype.subtypes[0])
        stype = stype.copy()
        stype.subtypes= (subtype,)
    
    return stype

